      PROGRAM CROLS
      IMPLICIT REAL*8(A-H,O-Z)
C
      CHARACTER  HEADA*70,HEADC*70,FINA*40,FINC*40,FOUT*40
      DIMENSION  A0(250,51),C0(250,51),U(17),Z(250),W(250)
C
C    PROGRAM CROLS PROVIDES A SIMULATED P-VALUE FOR TESTING A NESTED NULL
C       MODEL RELATIVE TO A FULL ALTERNATIVE MODEL FOR LSD REGRESSION
C
C          PRESENT SIZE LIMITS ON PROGRAM CROLS ARE THE FOLLOWING:
C           M = 250 OBSERVED CASES & N = 50 INDEPENDENT VARIABLES
C
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,10) '*** PROGRAM CROLS ***'
   10 FORMAT(T19,A)
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA FOR NULL MODEL:'
      READ(*,12) FINA
   12 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA FOR ALTERNATIVE MODEL:'
      READ(*,12) FINC
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,12) FOUT
      WRITE(*,*) 'ENTER # OF SIMULATIONS FOR TESTING NULL MODEL:'
      READ(*,*) MS
      TS = ONE*MS
      WRITE(*,*) 'ENTER RANDOM # GENERATOR SEED (ANY NON-ZERO INTEGER):'
      READ(*,*) ISEED
      OPEN(4,FILE=FINA)
      OPEN(5,FILE=FINC)
      OPEN(7,FILE=FOUT,STATUS='NEW')
C
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 100 II = 1,17
        S = 0.0D0
        T = 0.5D0
        DO 99 JJ = 1,K
          M1 = MOD(MOD(I1*J1,179)*K1,179)
          I1 = J1
          J1 = K1
          K1 = M1
          L1 = MOD(53*L1+1,169)
          IF(MOD(L1*M1,64).GE.32) S = S+T
          T = T/2
   99   CONTINUE
        U(II) = S
  100 CONTINUE
C
      READ(4,12) HEADA
      READ(4,*) MA,NA
      NA1 = NA+1
      READ(4,*) ((A0(I,J),J = 1,NA1),I = 1,MA)
      CLOSE(4)
      READ(5,12) HEADC
      READ(5,*) MC,NC
      NC1 = NC+1
      READ(5,*) ((C0(I,J),J = 1,NC1),I = 1,MC)
      CLOSE(5)
      IF(MA.EQ.MC) THEN
         M = MA
      ELSE
         STOP 'SAMPLE SIZES OF NULL AND ALTERNATIVE MODELS NOT EQUAL'
      END IF
C
      WRITE(7,1030)
 1030 FORMAT(T20,'*** OUTPUT OF CROLS ANALYSIS ***')
      WRITE(7,1035) HEADA
 1035 FORMAT(T2,'NULL DATA: ',A)
      WRITE(7,1040) HEADC
 1040 FORMAT(T2,'ALTERNATIVE DATA: ',A)
C
      CALL R2(M,NA,A0,Z,SA)
      CALL R2(M,NC,C0,W,SC)
      TOBS = (SC/SA)*(ONE+1.0D-11)
      WRITE(7,1090) 'VALUE OF OBSERVED STATISTIC (SS1/SS0) = ',TOBS
 1090 FORMAT(T6,A,G16.8)
C
      SS = ZERO
      DO 2000 L = 1,MS
         CALL SAMPWOR(M,M,U,Z)
         DO 1250 I = 1,M
            A0(I,NA1) = Z(I)
            C0(I,NC1) = Z(I)
 1250    CONTINUE
         CALL R2(M,NA,A0,W,SA)
         CALL R2(M,NC,C0,W,SC)
         T = SC/SA
         IF(T.LE.TOBS) SS = SS+ONE
 2000 CONTINUE
      PVALUE = SS/TS
      WRITE(7,2100) 'RANDOM # GENERATOR SEED = ',ISEED
 2100 FORMAT(T6,A,I8)
      WRITE(7,2100) 'NUMBER OF SIMULATIONS OF TEST STATISTIC = ',MS
      WRITE(7,1090) 'ESTIMATED P-VALUE FOR NESTED LSD TEST = ',PVALUE
      END
C
C
      SUBROUTINE R2(M,N,A,E,RSS)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  A(250,51),X(50),E(250),B(50,50),C(50),Y(250),YH(250),
     +           KP(50),KR(50),KC(50)
C
      ZERO = 0.0D0
      ND =  50
      DO 100 I = 1,M
         Y(I) = A(I,N+1)
  100 CONTINUE
      DO 300 I = 1,N
         C(I) = ZERO
         DO 200 K  = 1,M
            C(I) = C(I)+Y(K)*A(K,I)
  200    CONTINUE
  300 CONTINUE
      DO 600 I = 1,N
         DO 500 J = I,N
            B(I,J) = ZERO
            DO 400 K = 1,M
               B(I,J) = B(I,J)+A(K,I)*A(K,J)
  400       CONTINUE
            B(J,I) = B(I,J)
  500    CONTINUE
  600 CONTINUE
C
      CALL INVERT(N,B,KP,KR,KC,ND)
C
      DO 800 I = 1,N
         X(I) = ZERO
         DO 700 J = 1,N
            X(I) = X(I)+C(J)*B(I,J)
  700    CONTINUE
  800 CONTINUE
      RSS = ZERO
      DO 2300 I = 1,M
         YH(I) = ZERO
         DO 2200 J = 1,N
            YH(I) = YH(I)+X(J)*A(I,J)
 2200    CONTINUE
         E(I) = Y(I)-YH(I)
         RSS = RSS+E(I)**2
 2300 CONTINUE
      RETURN
      END
C
C
      SUBROUTINE INVERT(N,W,KP,KR,KC,ND)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  KP(N),KR(N),KC(N),W(ND,N)
C
      ZERO = 0.0D0
      DO 5 I = 1,N
         KP(I) = 0
    5 CONTINUE
      DO 45 I = 1,N
         T = ZERO
         DO 20 J = 1,N
            IF(KP(J).NE.1) THEN
               DO 15 K = 1,N
                  IF(KP(K)-1) 10,15,60
   10             IF(T.LT.DABS(W(J,K))) THEN
                     IR = J
                     IC = K
                     T = DABS(W(J,K))
                  END IF
   15          CONTINUE
            END IF
   20    CONTINUE
         KP(IC) = KP(IC)+1
         IF(IR.NE.IC) THEN
            DO 25 L = 1,N
               T = W(IR,L)
               W(IR,L) = W(IC,L)
               W(IC,L) = T
   25       CONTINUE
         END IF
         KR(I) = IR
         KC(I) = IC
         T = W(IC,IC)
         W(IC,IC) = 1.0D0
         DO 30 L = 1,N
            W(IC,L) = W(IC,L)/T
   30    CONTINUE
         DO 40 M = 1,N
            IF(M.NE.IC) THEN
               T = W(M,IC)
               W(M,IC) = ZERO
               DO 35 L = 1,N
                  W(M,L) = W(M,L)-W(IC,L)*T
   35          CONTINUE
            END IF
   40    CONTINUE
   45 CONTINUE
      DO 55 I = 1,N
         J = N-I+1
         IF(KR(J).NE.KC(J)) THEN
            K = KR(J)
            L = KC(J)
            DO 50 M = 1,N
               T = W(M,K)
               W(M,K) = W(M,L)
               W(M,L) = T
   50       CONTINUE
         END IF
   55 CONTINUE
   60 RETURN
      END
C
C
      SUBROUTINE SAMPWOR(M,N,U,X)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17),X(252)
C
C         SUBROUTINE SAMPWOR DRAWS A SAMPLE OF SIZE N FROM
C            A POPULATION OF SIZE M WITHOUT REPLACEMENT
C
C     M: SIZE OF POPULATION
C     N: SIZE OF SAMPLE DRAWN WITHOUT REPLACEMENT (1 </= N </= M)
C     X: REAL INPUT WHICH CONTAINS X(1),...,X(M) POPULATION VALUES
C     UNI(U): UNIFORM RANDOM NUMBER GENERATOR ON (0,1)
C
      DO 10 I = 1,N
         J = INT(I+(M-I+1)*UNI(U))
         BIN = X(J)
         X(J) = X(I)
         X(I) = BIN
   10 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
