      PROGRAM EGSLT
      IMPLICIT REAL*8(A-H,O-Z)
C
C  THIS IS AN EXACT PROGRAM FOR THE G-SAMPLE FISHER-PITMAN TEST (IT = 0)
C  AND THE CLASSES OF LINEAR RANK TESTS (IT = 1).  WHEN IT = 1, THEN THE
C  CHOICE OF KT = 1, 2 OR 3 SELECTS THE CLASS OF A(M,H), B(M,H) OR C(M,H)
C  G-SAMPLE RANK TESTS, RESPECTIVELY (SEE J. AMER. STATIST. ASSOC., VOL. 67,
C  PP. 850-854 FOR A DESCRIPTION OF THESE LINEAR RANK TESTS WHICH INCLUDE
C  THE MEDIAN, KRUSKAL-WALLIS, TAHA-L, ANSARI-BRADLEY, MOOD AND OTHER RANK
C  TESTS).  EACH G-SAMPLE RANK TEST IS ADJUSTED FOR TIED VALUES.  LET M BE
C  THE TOTAL NUMBER OF UNIVARIATE VALUES, AND LET N(I) BE THE NUMBER OF VALUES
C  IN THE ITH OF G GROUPS.  THE FIRST LINE CONSISTS OF THE ORDERED VALUES
C  M, G, IT, KT AND H, RESPECTIVELY.  THE SECOND LINE CONTAINS THE GROUP SIZES
C  N(1),...,N(G).  THEN THE NEXT M LINES CONSIST OF THE M UNIVERIATE VALUES
C  WITH THE N(1) VALUES OF THE 1ST GROUP FOLLOWED BY THE N(2) VALUES OF THE
C  2ND GROUP,..., AND FINALLY THE N(G) VALUES OF THE GTH GROUP.  THE EXACT,
C  ASYMPTOTIC AND ONE-WAY ANOVA P-VALUES ARE GIVEN FOR COMPARATIVE PURPOSES.
C
C  NOTE:  THIS PROGRAM IS CAPABLE OF MULTIPLE RUNS (MAKE SURE THAT THERE ARE
C         NO SPACES BETWEEN THE MULTIPLE DATA SETS).
C
C         PROGRAM MODIFIED 5/3/2002
C
      CHARACTER  HEAD*70,FIN*40,FOUT*40
      DIMENSION  V(10),Y(50),X(50),R(50)
      INTEGER  G,B(11),CB(10)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PHI = 1.0D-12
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA:'
      READ(*,100) FIN
  100 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,100) FOUT
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
      DO 110 I = 1,11
         B(I) = 0
  110 CONTINUE
  130 READ(3,*,END=160) M,G,IT,KT,H
      READ(3,*,END=160) (B(I),I = 1,G)
      DO 140 I = 1,M
         READ(3,*,END=150) Y(I)
         X(I) = Y(I)
  140 CONTINUE
      GO TO 170
  150 WRITE(*,*) 'ERROR:  END-OF FILE ENCOUNTERED'
  160 GO TO 2000
  170 V1 = ZERO
      V2 = ZERO
      V3 = ZERO
      DO 175 I = 1,M
         V1 = V1+Y(I)
         V2 = V2+Y(I)**2
  175 CONTINUE
      CB(1) = B(1)
      V(1) = ZERO
      DO 180 I = 2,G
         CB(I) = CB(I-1)+B(I)
         V(I) = ZERO
  180 CONTINUE
      DO 185 I = 1,CB(1)
         V(1) = V(1)+Y(I)
  185 CONTINUE
      V3 = V(1)**2/B(1)
      DO 195 J = 2,G
         DO 190 I = CB(J-1)+1,CB(J)
            V(J) = V(J)+Y(I)
  190    CONTINUE
         V3 = V3+V(J)**2/B(J)
  195 CONTINUE
      DFN = ONE*(G-1)
      DFD = ONE*(M-G)
      FSTAT = ((V3-V1**2/M)/DFN)/((V2-V3)/DFD)
      IF(FSTAT.LT.1.0D-5) THEN
         PF = ONE
         GO TO 200
      END IF
      CALL FPV(FSTAT,DFN,DFD,PF)
  200 IF(IT.EQ.0) GO TO 600
C  M UNIVARIATE VALUES ARE REPLACED WITH TIE ADJUSTED RANK ORDER STATISTICS
      CL = 1.0D30
      DO 210 I = 1,M
         IF(Y(I).LT.CL) CL = Y(I)
  210 CONTINUE
      DO 215 I = 1,M
         Y(I) = Y(I)-CL+1
  215 CONTINUE
      A1 = ZERO
      A2 = ZERO
      A3 = ONE*M-0.1D0
      B1 = ZERO-1.0D30
      B2 = 1.0D30
  218 IF(A2.GT.A3) GO TO 222
      DO 219 I = 1,M
         IF(Y(I).GT.B1.AND.Y(I).LT.B2) B2 = Y(I)*(ONE+PHI)
  219 CONTINUE
      DO 220 I = 1,M
         W = DABS(ONE-Y(I)/B2)
         IF(W.LT.1.0D-11) A1 = A1+1
  220 CONTINUE
      A4 = A2+(A1+1)/2
      DO 221 I = 1,M
         W = DABS(ONE-Y(I)/B2)
         IF(W.LT.1.0D-11) R(I) = A4
  221 CONTINUE
      A2 = A2+A1
      A1 = ZERO
      B1 = B2
      B2 = 1.0D30
      GO TO 218
  222 Z = ONE*(M+1)/2
      DO 300 I = 1,M
         GO TO (225,230,235) KT
  225    X(I) = R(I)**H
         GO TO 300
  230    W = DABS(Z-R(I))
         X(I) = ZERO
         IF(W.LT.1.D-11) GO TO 300
         X(I) = DABS(R(I)-Z)**H
         GO TO 300
  235    W = DABS(Z-R(I))
         X(I) = ZERO
         IF(W.LT.1.D-11) GO TO 300
         IF(R(I).GT.Z) THEN
            X(I) = (R(I)-Z)**H
         ELSEIF(R(I).LT.Z) THEN
            X(I) = ZERO-(Z-R(I))**H
         ENDIF
  300 CONTINUE
  600 SUM1 = ZERO
      SUM2 = ZERO
      DO 700 I = 1,M
         SUM1 = SUM1+X(I)
         SUM2 = SUM2+X(I)**2
  700 CONTINUE
      CALL PERM(M,G,B,MS,MP,TKW,X)
      P = ONE*MP/MS
      TG = (TKW-SUM1**2/M)*(ONE*M*(M-1)/(SUM2*M-SUM1**2))
      IF(IT.EQ.0) THEN
         HEAD = 'EXACT VERSION OF G-SAMPLE FISHER-PITMAN TEST'
         WRITE(6,800) HEAD
         GO TO 1100
      ENDIF
  800 FORMAT(T12,A)
      IF(IT.EQ.1) THEN
         GO TO (810,820,830) KT
  810    HEAD = 'EXACT VERSION OF G-SAMPLE A(M,H) RANK TEST'
         WRITE(6,800) HEAD
         WRITE(6,900) 'VALUE OF H FOR A(M,H) RANK TEST =',H
         GO TO 850
  820    HEAD = 'EXACT VERSION OF G-SAMPLE B(M,H) RANK TEST'
         WRITE(6,800) HEAD
         WRITE(6,900) 'VALUE OF H FOR B(M,H) RANK TEST =',H
         GO TO 850
  830    HEAD = 'EXACT VERSION OF G-SAMPLE C(M,H) RANK TEST'
         WRITE(6,800) HEAD
         WRITE(6,900) 'VALUE OF H FOR C(M,H) RANK TEST =',H
  850 ENDIF
  900 FORMAT(T2,A,G16.8)
 1100 WRITE(6,1200) 'INPUT HAS ',M,' UNIVARIATE OBSERVATIONS'
 1200 FORMAT(T2,A,I5,A)
      WRITE(6,1300) 'THERE ARE ',G, ' GROUPS OF SIZES ',
     +              (B(I),I = 1,G)
 1300 FORMAT(T2,A,I2,A,10I5)
      WRITE(6,1400) 'TOTAL NUMBER OF CONFIGURATIONS =',MS
 1400 FORMAT(T2,A,I9)
      WRITE(6,900) 'THE OBSERVED TEST STATISTIC =',TKW
      WRITE(6,900) 'STANDARDIZED CHI-SQUARED TEST STATISTIC =',TG
      WRITE(6,900) 'ONE-WAY ANOVA TEST STATISTIC =',FSTAT
      CALL CHISQ(TG,G-1,PA)
      WRITE(6,900) 'UPPER TAIL EXACT P-VALUE =',P
      WRITE(6,900) 'UPPER TAIL ASYMPTOTIC P-VALUE =',PA
      WRITE(6,900) 'UPPER TAIL ONE-WAY ANOVA P-VALUE =',PF
      WRITE(6,1500)
 1500 FORMAT('********************************************************')
      GO TO 130
 2000 CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE PERM(N,G,B,MS,MP,TKW,X)
C
      DOUBLE PRECISION  TKW,X(50)
      INTEGER  G,A(11,50),B(11),P(50)
C
      K = G
      IF(B(G+1).NE.0) K = K+1
      L = N
      DO 50 I = 1,K-1
         DO 30 J = 1,B(I)
            A(I,J) = I
   30    CONTINUE
         DO 40 J = B(I)+1,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-B(I)
   50 CONTINUE
      MS = 0
      MP = 0
   60 I = K-1
      NMARKS = B(K)+B(I)
      IF(MS.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+B(I)
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL DVAL(G,B,P,MS,MP,TKW,X)
      MS = MS+1
      GO TO 60
      END
C
C
      SUBROUTINE DVAL(G,B,P,MS,MP,TKW,X)
C
      DOUBLE PRECISION  ZERO,ONE,PREC,T,TKW,S(10),X(50)
      INTEGER  G,B(11),P(50)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PREC = 1.0D-12
      T = ZERO
      K = 0
      DO 20 I = 1,G
         S(I) = ZERO
         DO 10 J = K+1,K+B(I)
            S(I) = S(I)+X(P(J))
   10    CONTINUE
         T = T+S(I)**2/B(I)
         K = K+B(I)
   20 CONTINUE
      IF(MS.EQ.0) TKW = T*(ONE-PREC)
      IF(T.GT.TKW) MP = MP+1
      RETURN
      END
C
C
      SUBROUTINE CHISQ(T,N,P)
      DOUBLE PRECISION T,ZERO,ONE,V,P,PI,A,B,E1,E2,E3,E4,E5,H,W
C
      ZERO = 0.0D0
      ONE = 1.0D0
      IF(T.LT.ZERO) THEN
         P = ONE
         GO TO 60
      END IF
      V = ONE*N
      IF(V/2-N/2.GT.0.1D0) GO TO 30
      P = DEXP(-T/2)
      IF(N.LT.3) GO TO 60
      K = (N-2)/2
      B = ONE
      DO 20 J = 1,K
         L = J
         A = ONE
         DO 10 I = 1,L
            A = A*T/(2*I)
   10    CONTINUE
         B = B+A
   20 CONTINUE
      P = P*B
      GO TO 60
   30 PI = DATAN(ONE)*4
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*DSQRT(T)+1)
      P = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*DEXP(-T/2)/DSQRT(PI/2)
      IF(N.LT.2) GO TO 60
      K = (N-1)/2
      B = ZERO
      DO 50 J = 1,K
         L = J
         A = ONE
         DO 40 I = 1,L
            A = A*T/(2*I-1)
   40    CONTINUE
         B = B+A
   50 CONTINUE
      P = P+B*DEXP(-T/2)/DSQRT(PI*T/2)
   60 RETURN
      END
C
C
      SUBROUTINE FPV(X,DN,DD,P)
      DOUBLE PRECISION  ONE,X,Y,DN,DD,A,B,P,BETAIN
C
      ONE = 1.0D0
      A = DN/2
      B = DD/2
      Y = A*X/(A*X+B)
      P = ONE - BETAIN(Y,A,B)
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION BETAIN(X, A, B)
      DOUBLE PRECISION  X,A,B,ONE,TOL,AB,CX,XX,AA,BB,TERM,AI,RX,TEMP,
     +       DLNBTA
      LOGICAL INDEX
      ONE = 1.0D0
      TOL = 0.1D-7
      BETAIN = X
      AB = A + B
      CX = ONE - X
      IF (A .GE. AB * X) GO TO 10
      XX = CX
      CX = X
      AA = B
      BB = A
      INDEX = .TRUE.
      GO TO 20
   10 XX = X
      AA = A
      BB = B
      INDEX = .FALSE.
   20 TERM = ONE
      AI = ONE
      BETAIN = ONE
      NS = IDINT(BB + CX * AB)
      RX = XX / CX
   30 TEMP = BB - AI
      IF (NS .EQ. 0) RX = XX
   40 TERM = TERM * TEMP * RX / (AA + AI)
      BETAIN = BETAIN + TERM
      TEMP = DABS(TERM)
      IF (TEMP .LE. TOL .AND. TEMP .LE. TOL * BETAIN) GO TO 50
      AI = AI + 1
      NS = NS - 1
      IF (NS .GE. 0) GO TO 30
      TEMP = AB
      AB = AB + 1
      GO TO 40
   50 BETAIN = BETAIN * DEXP(AA * DLOG(XX) + (BB - 1) * DLOG(CX) -
     +         DLNBTA(AA, BB)) / AA
      IF (INDEX) BETAIN = ONE - BETAIN
      END
C
      DOUBLE PRECISION FUNCTION DLNBTA(A, B)
      DOUBLE PRECISION  A,B,DLNGAM
      DLNBTA = DLNGAM(A) + DLNGAM(B) - DLNGAM(A + B)
      END
C
C
      DOUBLE PRECISION FUNCTION DLNGAM(X)
      DOUBLE PRECISION  X,ONE,Z,D,PI,F
      ONE  = 1.0D0
      Z  = X - 1
      D  = ONE
      PI = DATAN(ONE)*4
      F  = Z + 10
      DO 10 J = 1, 9
         D = D * (Z + J)
   10 CONTINUE
      DLNGAM = (F * 2 - 1) * DLOG(F) / 2 - F + DLOG(PI * 2) / 2
     +          - DLOG(D) + ONE / (F * 12) - ONE / (F**3 * 360)
     +          + ONE / (F**5 * 1260) - ONE / (F**7 * 1680)
     +          + ONE / (F**9 * 1188)
      END
