      PROGRAM MRPP
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THE TEST STATISTIC ASSOCIATED WITH MULTI-RESPONSE PERMUTATION
C         PROCEDURES (MRPP) IS CALCULATED IN THIS FORTRAN-77 PROGRAM.
C         THIS TEST IS DESIGNED TO ANALYZE MULTIVARIATE DATA AT THE ORDINAL
C         (OR HIGHER) LEVELS AND IS BASED ON THE AVERAGE STANDARDIZED
C         EUCLIDEAN DISTANCE BETWEEN OBJECT POSITIONS WITHIN A PRIORI
C         DISJOINT GROUPS OF OBJECTS IN A FINITE POPULATION WHERE EACH
C         POSITION IS A POINT IN AN R-DIMENSIONAL SPACE (THUS THE ARGUMENTS
C         OF EACH R-DIMENSIONAL POINT CORRESPOND TO THE R MEASURED RESPONSES
C         OF EACH OBJECT IN THE FINITE POPULATION OF OBJECTS).  THIS PROGRAM
C         ALSO CONTAINS THE TRUNCATED DISTANCE OPTION (THE UPPER TRUNCATION
C         CONSTANT IS TERMED SUP).  THE PRESENT MAXIMUM VALUES OF G, N AND
C         R IN THIS PROGRAM ARE 40, 1800 AND 50, RESPECTIVELY.  THIS PROGRAM
C         CAN INVOKE EITHER AVERAGE PAIR-WISE DISTANCE FUNCTION (D.F.)
C         COMMENSURATION ON EACH RESPONSE OR A HOTELLING-TYPE COMMENSURATION.
C         THE A(N,H), B(N,H) AND C(N,H) RANKS TESTS ARE AN ADDITIONAL OPTION.
C         THIS PROGRAM IS CAPABLE OF MULTIPLE RUNS.
C
C         PROGRAM MODIFIED 12/15/2007
C
C       EITHER OF THE FOUR FOLLOWING WEIGHTING FORMS MAY BE CHOSEN.
C            CFORM = 1:  C(I) = n(I)/sum(n(I))
C            CFORM = 2:  C(I) = (n(I)-1)/sum(n(I)-1)
C            CFORM = 3:  C(I) = 1/sum(1)
C            CFORM = 4:  C(I) = n(I)*(n(I)-1)/sum(n(I)*(n(I)-1))
C       CFORM IS PASSED AS THE FOURTH PARAMETER ON THE SECOND RECORD
C       IN THE DATA FILE.  WEIGHTS FOR EACH GROUP ARE CALCULATED IN
C       SUBROUTINE DIST, AND STORED IN ARRAY C(I).
C
      CHARACTER HEAD*80,FIN*40,FOUT*40
      DIMENSION C(40),DATA(1800,51),RD(1800,50),ITHGRP(40),XI(40),AD(50)
      INTEGER   CFORM
C
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA'
      READ(*,100) FIN
  100 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,100) FOUT
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
  110 READ(3,100,END=120) HEAD
      GO TO 130
  120 GO TO 2000
  130 READ(3,*) NCASES,ID,V,SUP,LZ,IR,KT,H,IC,CFORM,NGRPS
      IF(NCASES.LT.6) STOP 'TOTAL NUMBER OF CASES < 6'
      READ(3,*) (ITHGRP(I),I = 1,NGRPS)
      L = 1
      M = ITHGRP(L)
      DO 160 I = 1,NCASES
         READ(3,*,END=170) (DATA(I,J),J = 1,ID)
  140    IF(I.LE.M) GO TO 150
         L = L+1
         IF(L.LE.NGRPS) THEN
            M = M+ITHGRP(L)
         ELSE
            M = NCASES
         END IF
         GO TO 140
  150    DATA(I,ID+1) = ONE*L
  160 CONTINUE
      IF(ID.EQ.1) LZ = 0
      GO TO 180
  170 WRITE(*,175) 'ERROR:  END OF FILE ENCOUNTERED'
  175 FORMAT(/,T3,A)
      STOP
  180 KZ = 0
      DO 200 I = 1,NGRPS
         KZ = KZ+ITHGRP(I)
  200 CONTINUE
      IF(KZ.GT.NCASES) STOP 'SUM OF GROUP SIZES EXCEEDS TOTAL CASES'
      IF(IR.NE.1) GO TO 1080
      DO 270 J = 1,ID
         CL = 1.0D30
         DO 210 I = 1,NCASES
            IF(DATA(I,J).LT.CL) CL = DATA(I,J)
  210    CONTINUE
         DO 220 I = 1,NCASES
            DATA(I,J) = DATA(I,J)-CL+1
  220    CONTINUE
         PHI = ONE+1.0D-12
         A1 = ZERO
         A2 = ZERO
         A3 = ONE*NCASES-0.1D0
         B1 = ZERO-1.0D30
         B2 = 1.0D30
  230    IF(A2.GT.A3) GO TO 270
         DO 240 I = 1,NCASES
            IF(DATA(I,J).GT.B1.AND.DATA(I,J).LT.B2) B2 = DATA(I,J)*PHI
  240    CONTINUE
         DO 250 I = 1,NCASES
            W = DABS(ONE-DATA(I,J)/B2)
            IF(W.LT.1.0D-11) A1 = A1+1
  250    CONTINUE
         A4 = A2+(A1+1)/2
         DO 260 I = 1,NCASES
            W = DABS(ONE-DATA(I,J)/B2)
            IF(W.LT.1.0D-11) RD(I,J) = A4
  260    CONTINUE
         A2 = A2+A1
         A1 = ZERO
         B1 = B2
         B2 = 1.0D30
         GO TO 230
  270 CONTINUE
      Z = ONE*(NCASES+1)/2
      DO 290 I = 1,NCASES
         DO 280 J = 1,ID
            GO TO (272,274,276) KT
  272       DATA(I,J) = RD(I,J)**H
            GO TO 280
  274       DATA(I,J) = ZERO
            W = DABS(Z-RD(I,J))
            IF(W.LT.1.0D-11) GO TO 280
            DATA(I,J) = DABS(RD(I,J)-Z)**H
            GO TO 280
  276       DATA(I,J) = ZERO
            W = DABS(Z-RD(I,J))
            IF(W.LT.1.0D-11) GO TO 280
            IF(RD(I,J).GT.Z) THEN
               DATA(I,J) = (RD(I,J)-Z)**H
            ELSEIF(RD(I,J).LT.Z) THEN
               DATA(I,J) = ZERO-(Z-RD(I,J))**H
            ENDIF
  280    CONTINUE
  290 CONTINUE
 1080 IF(ID.EQ.1.OR.IR.EQ.1.OR.LZ.EQ.1.OR.IC.NE.1) GO TO 1097
      DO 1090 I = 1,ID
         AD(I) = ZERO
         DO 1088 J = 1,NCASES-1
            DO 1086 K = J+1,NCASES
               AD(I) = AD(I)+(DABS(DATA(J,I)-DATA(K,I)))**V
 1086       CONTINUE
 1088    CONTINUE
         AD(I) = AD(I)**(ONE/V)
 1090 CONTINUE
      DO 1095 I = 1,NCASES
         DO 1093 J = 1,ID
            DATA(I,J) = DATA(I,J)/AD(J)
 1093    CONTINUE
 1095 CONTINUE
 1097 CALL DIST(NCASES,ID,V,SUP,CFORM,NGRPS,ITHGRP,DATA,C,XI,
     +   DBAR,D1,VAR,T,GAM1,PROB,LZ)
      RHO = 1.0D0-DBAR/D1
      WRITE(6,100) HEAD
      IF(LZ.EQ.1) THEN
         HEAD = 'MULTIVARIATE HOTELLING-TYPE PERMUTATION TESTS'
      ELSE
         HEAD = 'MULTI-RESPONSE PERMUTATION PROCEDURES'
      END IF
      WRITE(6,1100) HEAD
 1100 FORMAT(T23,A,/)
      WRITE(6,1200) 'INPUT HAS ', NCASES,' OBSERVATIONS'
 1200 FORMAT(T10,A,I4,A)
      WRITE(6,1200) 'THERE WERE', ID, ' DIMENSION(S)'
      WRITE(6,1300) 'IN: ', NGRPS, ' GROUPS OF SIZES ',
     +              (ITHGRP(I),I = 1,NGRPS)
 1300 FORMAT(T10,A,I2,A,10I5)
      GO TO (1410,1420,1430,1440) CFORM
 1410 WRITE(6,1450) 'C(I) = n(I)/sum(n(I))'
      GO TO 1500
 1420 WRITE(6,1450) 'C(I) = (n(I)-1)/sum(n(I)-1)'
      GO TO 1500
 1430 WRITE(6,1450) 'C(I) = 1/sum(1)'
      GO TO 1500
 1440 WRITE(6,1450) 'C(I) = n(I)*(n(I)-1)/sum(n(I)*(n(I)-1))'
 1450 FORMAT(T10,A)
 1500 WRITE(6,100) ' '
      DO 1700 I = 1,NGRPS
         WRITE(6,1600) 'GROUP NUMBER ', I, ' OF SIZE ', ITHGRP(I),
     +                 ' HAS AN AVERAGE DISTANCE =', XI(I)
 1600    FORMAT(T10,A,I2,A,I4,A,G16.8)
 1700 CONTINUE
      WRITE(6,100) ' '
      IF(IR.EQ.1) THEN
         GO TO (1725,1750,1775) KT
 1725    HEAD = 'MRPP VERSION OF A(N,H) RANKS TEST'
         WRITE(6,1450) HEAD
         WRITE(6,1800) 'A(N,H) RANKS TEST EXPONENT H = ',H
         GO TO 1825
 1750    HEAD = 'MRPP VERSION OF B(N,H) RANKS TEST'
         WRITE(6,1450) HEAD
         WRITE(6,1800) 'B(N,H) RANKS TEST EXPONENT H = ',H
         GO TO 1825
 1775    HEAD = 'MRPP VERSION OF C(N,H) RANKS TEST'
         WRITE(6,1450) HEAD
         WRITE(6,1800) 'C(N,H) RANKS TEST EXPONENT H = ',H
      END IF
 1800 FORMAT(T10,A,G16.8)
 1825 IF(LZ.EQ.0) THEN
         WRITE(6,1850) '1 (0) IMPLIES (NO) AVE. D.F. COMMENSURATION:',IC
      END IF
 1850 FORMAT(T10,A,I3)
      WRITE(6,1800) 'THE TEST STATISTIC IS =', T
      WRITE(6,1800) 'THE DISTANCE EXPONENT =', V
      WRITE(6,1800) 'THE UPPER BOUND VALUE =', SUP
      WRITE(6,1800) 'THE OBSERVED DELTA IS =', DBAR
      WRITE(6,1800) 'THE EXPECTED DELTA IS =', D1
      WRITE(6,1800) 'THE VARIANCE OF DELTA =', VAR
      WRITE(6,1800) 'THE SKEWNESS OF DELTA =', GAM1
      WRITE(6,1800) 'THE AGREEMENT MEASURE =', RHO
      WRITE(6,1800) 'P-VALUE OF A SMALLER OR EQUAL DELTA =', PROB
      WRITE(6,1875)
 1875 FORMAT('********************************************************')
      GO TO 110
 2000 CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE DIST(NCASES,ID,V,SUP,CFORM,NGRPS,ITHGRP,DATA,C,XI,
     +   DBAR,D1,VAR,T,GAM1,PROB,LZ)
      IMPLICIT REAL*8(A-H,O-Z)
C
C         SUBROUTINE DIST CALCULATES THE DISTANCES BETWEEN THE DATA
C         POINTS, PLUS THE PARAMETERS WHICH ARE DIRECTLY DEPENDENT
C         ON THE DISTANCES.
C
      DIMENSION  B(12),C(40),DATA(1800,51),DC(3,1800),DEL(1800,1800),
     +   ITHGRP(40),Q(6),R(40,6),S(11),XI(40),U(50),Y(50,50),
     +   KP(50),KR(50),KC(50)
      INTEGER  CFORM
C
      ND = 50
      ZERO = 0.0D0
      ONE = 1.0D0
      DBAR = ZERO
      DO 10 I = 1,NGRPS
         XI(I) = ZERO
   10 CONTINUE
      IF(LZ.EQ.1) GO TO 50
      DO 40 I = 1,NCASES
         DO 30 J = 1,NCASES
            DEL(I,J) = ZERO
            DO 20 K = 1,ID
               DEL(I,J) = DEL(I,J)+(DATA(I,K)-DATA(J,K))**2
   20       CONTINUE
            DEL(I,J) = DEL(I,J)**(V/2)
            IF(DEL(I,J).GT.SUP) DEL(I,J) = SUP
            IS = IDNINT(DATA(I,ID+1))
            JS = IDNINT(DATA(J,ID+1))
            IF(IS.LE.NGRPS.AND.IS.EQ.JS) XI(IS) = XI(IS)+DEL(I,J)
   30    CONTINUE
   40 CONTINUE
      GO TO 1405
   50 DO 200 I = 1,ID
         U(I) = ZERO
         DO 100 J = 1,NCASES
            U(I) = U(I)+DATA(J,I)
  100    CONTINUE
         U(I) = U(I)/NCASES
  200 CONTINUE
      DO 500 I = 1,ID
         DO 400 J = I,ID
            Y(I,J) = ZERO
            DO 300 K = 1,NCASES
               Y(I,J) = Y(I,J)+(DATA(K,I)-U(I))*(DATA(K,J)-U(J))
  300       CONTINUE
            Y(J,I) = Y(I,J)
  400    CONTINUE
  500 CONTINUE
      CALL INVERT(ID,Y,KP,KR,KC,ND)
      DO 1400 I = 1,NCASES
         DO 1300 J = 1,NCASES
            DEL(I,J) = ZERO
            DO 1200 K = 1,ID
               DO 1100 L = 1,ID
                  DEL(I,J) = DEL(I,J)+(DATA(I,K)-DATA(J,K))*Y(K,L)*
     1                                (DATA(I,L)-DATA(J,L))
 1100          CONTINUE
 1200       CONTINUE
            DEL(I,J) = DEL(I,J)**(V/2)
            IF(DEL(I,J).GT.SUP) DEL(I,J) = SUP
            IS = IDNINT(DATA(I,ID+1))
            JS = IDNINT(DATA(J,ID+1))
            IF(IS.LE.NGRPS.AND.IS.EQ.JS) XI(IS) = XI(IS)+DEL(I,J)
 1300    CONTINUE
 1400 CONTINUE
 1405 GO TO (1410,1430,1450,1470) CFORM
 1410 FK = ZERO
      DO 1415 I = 1,NGRPS
         FK = FK+ITHGRP(I)
 1415 CONTINUE
      DO 1420 I = 1,NGRPS
         C(I) = ONE*ITHGRP(I)/FK
 1420 CONTINUE
      GO TO 1490
 1430 FK = ZERO
      DO 1435 I = 1,NGRPS
         FK = FK+ITHGRP(I)
 1435 CONTINUE
      DO 1440 I = 1,NGRPS
         C(I) = ONE*(ITHGRP(I)-1)/(FK-NGRPS)
 1440 CONTINUE
      GO TO 1490
 1450 DO 1460 I = 1,NGRPS
         C(I) = ONE/NGRPS
 1460 CONTINUE
      GO TO 1490
 1470 W = ZERO
      DO 1475 I = 1,NGRPS
         W = W+ONE*ITHGRP(I)*(ITHGRP(I)-1)
 1475 CONTINUE
      DO 1480 I = 1,NGRPS
         C(I) = ONE*ITHGRP(I)*(ITHGRP(I)-1)/W
 1480 CONTINUE
 1490 DO 1500 I = 1,NGRPS
         XI(I) = XI(I)/(ITHGRP(I)*(ITHGRP(I)-1))
         DBAR = DBAR+C(I)*XI(I)
 1500 CONTINUE
      DO 2150 I = 1,NCASES
         DO 2140 K = 1,3
            DC(K,I) = ZERO
            DO 2130 J = 1,NCASES
               DC(K,I) = DC(K,I)+DEL(I,J)**K
 2130       CONTINUE
 2140    CONTINUE
 2150 CONTINUE
      DO 2160 I = 1,12
         B(I) = ZERO
 2160 CONTINUE
      DO 2170 I = 1,NCASES
         B(1) = B(1)+DC(1,I)
         B(2) = B(2)+DC(2,I)
         B(3) = B(3)+DC(1,I)**2
         B(5) = B(5)+DC(3,I)
         B(6) = B(6)+DC(1,I)*DC(2,I)
         B(10) = B(10)+DC(1,I)**3
 2170 CONTINUE
      DO 2190 I = 1,NCASES-1
         DO 2180 J = I+1,NCASES
            B(9) = B(9)+DC(1,I)*DC(1,J)*DEL(I,J)
 2180    CONTINUE
 2190 CONTINUE
      B(9) = B(9)*2
      DO 2240 I = 1,NCASES-2
         DO 2230 J = I+1,NCASES-1
            DO 2220 K = J+1,NCASES
               SUM = DEL(I,J)*DEL(I,K)*DEL(J,K)
               B(7) = B(7)+SUM
 2220       CONTINUE
 2230    CONTINUE
 2240 CONTINUE
      B(7) = B(7)*6
      B(4) = B(1)**2
      B(8) = B(1)*B(2)
      B(11) = B(1)*B(3)
      B(12) = B(1)**3
      A1 = B(1)
      A2 = B(2)
      A3 = B(3)-B(2)
      A4 = B(4)-A2*2-A3*4
      A5 = B(5)
      A6 = B(6)-B(5)
      A7 = B(7)
      A8 = B(8)+(B(5)-B(6)*2)*2
      A9 = B(9)+B(5)-B(6)*2-B(7)
      A10 = B(10)+B(5)*2-B(6)*3
      A11 = B(11)-(B(10)-B(7)+(B(5)+B(9))*2)*2+B(6)*10-B(8)
      A12 = B(12)-(A5+(A7+A10+(A6+A9)*3)*2)*4-(A8+A11*2)*6
      Q(1) = NCASES
      DO 2280 I = 2,6
         Q(I) = Q(I-1)*(NCASES-I+1)
 2280 CONTINUE
      D1 = A1/Q(2)
      D2 = A2/Q(2)
      D3 = A3/Q(3)
      D4 = A4/Q(4)
      D5 = A5/Q(2)
      D6 = A6/Q(3)
      D7 = A7/Q(3)
      D8 = A8/Q(4)
      D9 = A9/Q(4)
      D10 = A10/Q(4)
      D11 = A11/Q(5)
      D12 = A12/Q(6)
      DO 2300 I = 1,NGRPS
         R(I,1) = ITHGRP(I)
         DO 2290 J = 2,6
            R(I,J) = R(I,J-1)*(ITHGRP(I)-J+1)
 2290    CONTINUE
 2300 CONTINUE
      DO 2310 I = 1,11
         S(I) = ZERO
 2310 CONTINUE
      DO 2340 I = 1,NGRPS
         S(4) = S(4)+C(I)**2/R(I,2)
         S(5) = S(5)+C(I)**2/R(I,1)
         S(6) = S(6)+C(I)**3/R(I,2)**2
         S(7) = S(7)+C(I)**3*R(I,3)/R(I,2)**3
         S(8) = S(8)+C(I)**2*((ONE-C(I))/R(I,2)+C(I)*R(I,4)/R(I,2)**3)
         S(9) = S(9)+C(I)**3*R(I,4)/R(I,2)**3
         S(10) = S(10)+C(I)**2*((ONE-C(I))*R(I,3)/R(I,2)**2+
     1           C(I)*R(I,5)/R(I,2)**3)
         S(11) = S(11)+C(I)*((ONE-C(I))*(ONE-C(I)*2)+C(I)*(ONE-C(I))*
     1           R(I,4)*3/R(I,2)**2+C(I)**2*R(I,6)/R(I,2)**3)
 2340 CONTINUE
      VAR = (S(4)-ONE/Q(2))*(D2-D3*2+D4)*2+(S(5)-ONE/Q(1))*(D3-D4)*4
      STDEV = DSQRT(VAR)
      T = (DBAR-D1)/STDEV
      EDELT3 = S(6)*D5*4+S(7)*(D6*3+D7)*8+S(8)*D8*6+
     1         S(9)*(D9*3+D10)*8+S(10)*D11*12+S(11)*D12
      CUM3 = EDELT3-D1*VAR*3-D1**3
      GAM1 = CUM3/STDEV**3
      CALL PVALUE(T,GAM1,PROB)
      RETURN
      END
C
C
      SUBROUTINE INVERT(N,W,KP,KR,KC,ND)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  KP(N),KR(N),KC(N),W(ND,N)
C
      ZERO = 0.0D0
      DO 5 I = 1,N
         KP(I) = 0
    5 CONTINUE
      DO 45 I = 1,N
         T = ZERO
         DO 20 J = 1,N
            IF(KP(J).NE.1) THEN
               DO 15 K = 1,N
                  IF(KP(K)-1) 10,15,60
   10             IF(T.LT.DABS(W(J,K))) THEN
                     IR = J
                     IC = K
                     T = DABS(W(J,K))
                  END IF
   15          CONTINUE
            END IF
   20    CONTINUE
         KP(IC) = KP(IC)+1
         IF(IR.NE.IC) THEN
            DO 25 L = 1,N
               T = W(IR,L)
               W(IR,L) = W(IC,L)
               W(IC,L) = T
   25       CONTINUE
         END IF
         KR(I) = IR
         KC(I) = IC
         T = W(IC,IC)
         W(IC,IC) = 1.0D0
         DO 30 L = 1,N
            W(IC,L) = W(IC,L)/T
   30    CONTINUE
         DO 40 M = 1,N
            IF(M.NE.IC) THEN
               T = W(M,IC)
               W(M,IC) = ZERO
               DO 35 L = 1,N
                  W(M,L) = W(M,L)-W(IC,L)*T
   35          CONTINUE
            END IF
   40    CONTINUE
   45 CONTINUE
      DO 55 I = 1,N
         J = N-I+1
         IF(KR(J).NE.KC(J)) THEN
            K = KR(J)
            L = KC(J)
            DO 50 M = 1,N
               T = W(M,K)
               W(M,K) = W(M,L)
               W(M,L) = T
   50       CONTINUE
         END IF
   55 CONTINUE
   60 RETURN
      END
C
C
      SUBROUTINE PVALUE(T,GAM,PROB)
C
C         SUBROUTINE PVALUE CALCULATES THE PROBABILITY THAT A VALUE OF
C         T IS LESS THAN OR EQUAL TO THE OBSERVED VALUE OF T.
C
      DOUBLE PRECISION  T,GAM,PROB,ZERO,ONE,PI,A,B,C,D,E,F,G,H,R,
     +   U,W,X,Y,E1,E2,E3,E4,E5,G1,G2,G3,H0,H1,H1N,H2,H3
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      IF(DABS(GAM).LT.0.01D0) GO TO 50
      R = ONE*2/DABS(GAM)
      D = R*R
      F = R*R+10
      DO 10 I = 1,9
         D = D*(R*R+I)
   10 CONTINUE
      U = (F*2-1)*DLOG(F)/2-F+DLOG(PI*2)/2-DLOG(D)+ONE/(F*12)
     +        -ONE/(F*F*F*360)
      A = R*R-1
      B = R*R*(DLOG(R)-1)-U
      G1 = 0.0375D0
      G2 = 0.075D0
      G3 = 0.0125D0
      H1 = ZERO
      H2 = ZERO
      W = -1.99D0/GAM
      IF(GAM.LT.ZERO) GO TO 30
      IF(T.LT.W) THEN
         PROB = ZERO
         RETURN
      END IF
      X = T
      Y = T+15
      DO 20 I = 1,199
         G = A*DLOG(R+X+G1*(2*I-1))-R*(X+G1*(2*I-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R+X+G2*I)-R*(X+G2*I)+B
         H2 = H2+E(G)
   20 CONTINUE
      G = A*DLOG(R+X+G1*399)-R*(X+G1*399)+B
      H1N = E(G)
      G = A*DLOG(R+X)-R*X+B
      H0 = E(G)
      G = A*DLOG(R+Y)-R*Y+B
      H3 = E(G)
      PROB = ONE-G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   30 IF(T.GT.W) THEN
         PROB = ONE
         RETURN
      END IF
      X = T-15
      Y = T
      DO 40 I = 1,199
         G = A*DLOG(R-X-G1*(2*I-1))+R*(X+G1*(2*I-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R-X-G2*I)+R*(X+G2*I)+B
         H2 = H2+E(G)
   40 CONTINUE
      G = A*DLOG(R-X-G1*399)+R*(X+G1*399)+B
      H1N = E(G)
      G = A*DLOG(R-X)+R*X+B
      H0 = E(G)
      G = A*DLOG(R-Y)+R*Y+B
      H3 = E(G)
      PROB = G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   50 PROB = ZERO
      G = -T*T/2
      IF(G.LT.-709.1D0) GO TO 70
      A = 4.2D0
      U = DABS(T)
      IF(U.GT.A) GO TO 60
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*U+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 70
   60 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   70 IF(T.GT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
C
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF(G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END
