      PROGRAM MRSP
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THE TEST STATISTIC ASSOCIATED WITH MULTI-RESPONSE SEQUENCE
C         PERMUTATION PROCEDURES (MRSP) IS CALCULATED IN THIS FORTRAN-77
C         PROGRAM.  THIS TEST IS DESIGNED TO ANALYZE MULTI-RESPONSE DATA
C         AT THE ORDINAL (OR HIGHER) LEVELS AND IS BASED ON THE AVERAGE
C         EUCLIDEAN DISTANCE BETWEEN ADJACENT POINTS OF AN OBSERVED SEQUENCE
C         OF POINTS IN AN R-DIMENSIONAL SPACE (WHICH CORRESPOND TO R MEASURED
C         RESPONSES ON EACH OBJECT OF AN OBSERVED SEQUENCE OF OBJECTS).
C         THE PRESENT MAXIMUM VALUES OF N AND R IN THIS PROGRAM ARE 200
C         AND 50, RESPECTIVELY.  AVERAGE DISTANCE FUNCTION COMMENSURATION
C         IS AN ALTERNATIVE.  THIS PROGRAM IS CAPABLE OF MULTIPLE RUNS.
C
C         PROGRAM MODIFIED 8/21/2006
C
      CHARACTER  HEAD*80,FIN*40,FOUT*40
      DIMENSION  AD(50),DATA(200,51)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA'
      READ(*,1000) FIN
 1000 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,1000) FOUT
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
 1010 READ(3,1000,END=1020) HEAD
      GO TO 1030
 1020 GO TO 2000
 1030 READ(3,*) NCASES,ID,V,IC
      IF(NCASES.LT.6) STOP 'TOTAL NUMBER OF CASES < 6'
      DO 1040 I = 1,NCASES
         READ(3,*,END=1050) (DATA(I,J),J = 1,ID)
 1040 CONTINUE
      GO TO 1070
 1050 WRITE(*,1060) 'ERROR:  END OF FILE ENCOUNTERED'
 1060 FORMAT(/,T3,A)
      STOP
 1070 IF(ID.EQ.1) IC = 0
      IF(ID.EQ.1.OR.IC.NE.1) GO TO 1090
      DO 1076 I = 1,ID
         AD(I) = ZERO
         DO 1074 J = 1,N-1
            DO 1072 K = J+1,N
               AD(I) = AD(I)+(DABS(DATA(J,I)-DATA(K,I)))**V
 1072       CONTINUE
 1074    CONTINUE
         AD(I) = AD(I)**(ONE/V)
 1076 CONTINUE
      DO 1084 I = 1,N
         DO 1082 J = 1,ID
            DATA(I,J) = DATA(I,J)/AD(J)
 1082    CONTINUE
 1084 CONTINUE
 1090 CALL DIST(NCASES,ID,V,DATA,DBAR,D1,VAR,T,GAM1,PROB)
      RHO = 1.0D0-DBAR/D1
      WRITE(6,1000) HEAD
      HEAD = 'MULTI-RESPONSE PERMUTATION SEQUENCE PROCEDURES (MRSP)'
      WRITE(6,1100) HEAD
 1100 FORMAT(T23,A,/)
      WRITE(6,1200) 'INPUT HAS ', NCASES,' OBSERVATIONS'
 1200 FORMAT(T10,A,I3,A,/)
      WRITE(6,1200) 'THERE WERE', ID, ' DIMENSION(S)'
      WRITE(6,1500) IC
 1500 FORMAT(T10,'1 (0) IMPLIES (NO) AVE. D.F. COMMENSURATION:',I3)
      WRITE(6,1800) 'THE TEST STATISTIC IS =', T
 1800 FORMAT(T10,A,G16.8)
      WRITE(6,1800) 'THE DISTANCE EXPONENT =', V
      WRITE(6,1800) 'THE OBSERVED DELTA IS =', DBAR
      WRITE(6,1800) 'THE EXPECTED DELTA IS =', D1
      WRITE(6,1800) 'THE VARIANCE OF DELTA =', VAR
      WRITE(6,1800) 'THE SKEWNESS OF DELTA =', GAM1
      WRITE(6,1800) 'THE AGREEMENT MEASURE =', RHO
      WRITE(6,1800) 'P-VALUE OF A SMALLER OR EQUAL DELTA =', PROB
      WRITE(6,1850)
 1850 FORMAT('********************************************************')
      GO TO 1010
 2000 CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE DIST(NCASES,ID,V,DATA,DBAR,D1,VAR,T,GAM1,PROB)
      IMPLICIT REAL*8(A-H,O-Z)
C
C         SUBROUTINE DIST CALCULATES THE DISTANCES BETWEEN THE DATA
C         POINTS, PLUS THE PARAMETERS WHICH ARE DIRECTLY DEPENDENT
C         ON THE DISTANCES.
C
      DIMENSION  B(12),DATA(200,51),DC(3,200),DEL(200,200),Q(6)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      DBAR = ZERO
      DO 1400 I = 1,NCASES
         DO 1300 J = 1,NCASES
            DEL(I,J) = ZERO
            DO 1200 K = 1,ID
               DEL(I,J) = DEL(I,J)+(DATA(I,K)-DATA(J,K))**2
 1200       CONTINUE
            DEL(I,J) = DEL(I,J)**(V/2)
 1300    CONTINUE
 1400 CONTINUE
      DO 1600 I = 1,NCASES-1
         DBAR = DBAR+DEL(I,I+1)
 1600 CONTINUE
      DBAR = DBAR/(NCASES-1)
      DO 2150 I = 1,NCASES
         DO 2140 K = 1,3
            DC(K,I) = ZERO
            DO 2130 J = 1,NCASES
               DC(K,I) = DC(K,I)+DEL(I,J)**K
 2130       CONTINUE
 2140    CONTINUE
 2150 CONTINUE
      DO 2160 I = 1,12
         B(I) = ZERO
 2160 CONTINUE
      DO 2170 I = 1,NCASES
         B(1) = B(1)+DC(1,I)
         B(2) = B(2)+DC(2,I)
         B(3) = B(3)+DC(1,I)**2
         B(5) = B(5)+DC(3,I)
         B(6) = B(6)+DC(1,I)*DC(2,I)
         B(10) = B(10)+DC(1,I)**3
 2170 CONTINUE
      DO 2190 I = 1,NCASES-1
         DO 2180 J = I+1,NCASES
            B(9) = B(9)+DC(1,I)*DC(1,J)*DEL(I,J)
 2180    CONTINUE
 2190 CONTINUE
      B(9) = B(9)*2
      DO 2240 I = 1,NCASES-2
         DO 2230 J = I+1,NCASES-1
            DO 2220 K = J+1,NCASES
               SUM = DEL(I,J)*DEL(I,K)*DEL(J,K)
               B(7) = B(7)+SUM
 2220       CONTINUE
 2230    CONTINUE
 2240 CONTINUE
      B(7) = B(7)*6
      B(4) = B(1)**2
      B(8) = B(1)*B(2)
      B(11) = B(1)*B(3)
      B(12) = B(1)**3
      A1 = B(1)
      A2 = B(2)
      A3 = B(3)-B(2)
      A4 = B(4)-A2*2-A3*4
      A5 = B(5)
      A6 = B(6)-B(5)
      A7 = B(7)
      A8 = B(8)+(B(5)-B(6)*2)*2
      A9 = B(9)+B(5)-B(6)*2-B(7)
      A10 = B(10)+B(5)*2-B(6)*3
      A11 = B(11)-(B(10)-B(7)+(B(5)+B(9))*2)*2+B(6)*10-B(8)
      A12 = B(12)-(A5+(A7+A10+(A6+A9)*3)*2)*4-(A8+A11*2)*6
      Q(1) = NCASES
      DO 2280 I = 2,6
         Q(I) = Q(I-1)*(NCASES-I+1)
 2280 CONTINUE
      D1 = A1/Q(2)
      D2 = A2/Q(2)
      D3 = A3/Q(3)
      D4 = A4/Q(4)
      D5 = A5/Q(2)
      D6 = A6/Q(3)
      D7 = A7/Q(3)
      D8 = A8/Q(4)
      D9 = A9/Q(4)
      D10 = A10/Q(4)
      D11 = A11/Q(5)
      D12 = A12/Q(6)
      M = NCASES
      VAR = (D2*(M-1)-D3*(M-2)*2+D4*(M-3))*M*(M-2)/Q(2)**2
      STDEV = DSQRT(VAR)
      T = (DBAR-D1)/STDEV
      GAM1 = (D5*(M-1)*(M-2)-D6*(M-2)*(M-2)*6+D7*(M-2)*4+
     1       (D8*3+D10*4)*(M-2)*(M-3)+(D9-D11*2)*(M-3)*(M-4)*6+
     2       D12*(M-3)*(M-5)*4)*M*(M-4)/(STDEV*Q(2))**3
      CALL PVALUE(T,GAM1,PROB)
      RETURN
      END
C
C
      SUBROUTINE PVALUE(T,GAM,PROB)
C
C         SUBROUTINE PVALUE CALCULATES THE PROBABILITY THAT A VALUE OF
C         T IS LESS THAN OR EQUAL TO THE OBSERVED VALUE OF T.
C
      DOUBLE PRECISION  T,GAM,PROB,ZERO,ONE,PI,A,B,C,D,E,F,G,H,R,
     +   U,W,X,Y,E1,E2,E3,E4,E5,G1,G2,G3,H0,H1,H1N,H2,H3
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      IF(DABS(GAM).LT.0.01D0) GO TO 50
      R = ONE*2/DABS(GAM)
      D = R*R
      F = R*R+10
      DO 10 I = 1,9
         D = D*(R*R+I)
   10 CONTINUE
      U = (F*2-1)*DLOG(F)/2-F+DLOG(PI*2)/2-DLOG(D)+ONE/(F*12)
     +        -ONE/(F*F*F*360)
      A = R*R-1
      B = R*R*(DLOG(R)-1)-U
      G1 = 0.0375D0
      G2 = 0.075D0
      G3 = 0.0125D0
      H1 = ZERO
      H2 = ZERO
      W = -1.99D0/GAM
      IF(GAM.LT.ZERO) GO TO 30
      IF(T.LT.W) THEN
         PROB = ZERO
         RETURN
      END IF
      X = T
      Y = T+15
      DO 20 I = 1,199
         G = A*DLOG(R+X+G1*(2*I-1))-R*(X+G1*(2*I-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R+X+G2*I)-R*(X+G2*I)+B
         H2 = H2+E(G)
   20 CONTINUE
      G = A*DLOG(R+X+G1*399)-R*(X+G1*399)+B
      H1N = E(G)
      G = A*DLOG(R+X)-R*X+B
      H0 = E(G)
      G = A*DLOG(R+Y)-R*Y+B
      H3 = E(G)
      PROB = ONE-G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   30 IF(T.GT.W) THEN
         PROB = ONE
         RETURN
      END IF
      X = T-15
      Y = T
      DO 40 I = 1,199
         G = A*DLOG(R-X-G1*(2*I-1))+R*(X+G1*(2*I-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R-X-G2*I)+R*(X+G2*I)+B
         H2 = H2+E(G)
   40 CONTINUE
      G = A*DLOG(R-X-G1*399)+R*(X+G1*399)+B
      H1N = E(G)
      G = A*DLOG(R-X)+R*X+B
      H0 = E(G)
      G = A*DLOG(R-Y)+R*Y+B
      H3 = E(G)
      PROB = G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   50 PROB = ZERO
      G = -T*T/2
      IF(G.LT.-709.1D0) GO TO 70
      A = 4.2D0
      U = DABS(T)
      IF(U.GT.A) GO TO 60
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*U+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 70
   60 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   70 IF(T.GT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
C
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF(G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END
