      PROGRAM RGSLT
      IMPLICIT REAL*8(A-H,O-Z)
C
C  THIS IS A RESAMPLING PROGRAM FOR THE G-SAMPLE FISHER-PITMAN TEST (IT = 0)
C  AND THE CLASSES OF LINEAR RANK TESTS (IT = 1).  WHEN IT = 1, THEN THE
C  CHOICE OF KT = 1, 2 OR 3 SELECTS THE CLASS OF A(M,H), B(M,H) OR C(M,H)
C  G-SAMPLE RANK TESTS, RESPECTIVELY (SEE J. AMER. STATIST. ASSOC., VOL. 67,
C  PP. 850-854 FOR A DESCRIPTION OF THESE LINEAR RANK TESTS WHICH INCLUDE
C  THE MEDIAN, KRUSKAL-WALLIS, TAHA-L, ANSARI-BRADLEY, MOOD AND OTHER RANK
C  TESTS)1.  EACH G-SAMPLE RANK TEST IS ADJUSTED FOR TIED VALUES.  LET L
C  BE THE NUMBER OF RESAMPLING SIMULATIONS, LET M BE THE TOTAL NUMBER OF
C  UNIVARIATE VALUES.  LET N(I) BE THE NUMBER VALUES IN THE ITH OF G GROUPS.
C  THE FIRST LINE OF INPUT DATA FILE CONSISTS OF THE ORDERED VALUES M, G, IT,
C  KT AND H, RESPECTIVELY.  THE SECOND LINE CONTAINS THE G GROUP SIZES N(1),
C  ...,N(G).  THEN THE NEXT M LINES CONSIST OF THE M UNIVARIATE VALUES WITH
C  THE N(1) VALUES OF THE 1ST GROUP FOLLOWED BY THE N(2) VALUES OF THE 2ND
C  GROUP,..., AND FINALLY THE N(G) VALUES OF THE GTH GROUP.  THE EXACT,
C  ASYMPTOTIC AND ONE-WAY ANOVA P-VALUES ARE GIVEN FOR COMPARATIVE PURPOSES.
C
C  NOTE:  THIS PROGRAM IS CAPABLE OF MULTIPLE RUNS (MAKE SURE THAT THERE ARE
C         NO SPACES BETWEEN THE MULTIPLE DATA SETS).
C
C         PROGRAM MODIFIED 5/3/2002
C
      CHARACTER  HEAD*70,FIN*40,FOUT*40
      DIMENSION  U(17),N(10),NC(10),V(10),S1(10),Y(5000),X(5000),R(5000)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PHI = 1.0D-12
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA:'
      READ(*,100) FIN
  100 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,100) FOUT
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 104 II = 1,17
         S = 0.0D0
         T = 0.5D0
         DO 102 JJ = 1,K
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) S = S+T
            T = T/2
  102    CONTINUE
         U(II) = S
  104 CONTINUE
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
  130 READ(3,*,END=160) M,IG,IT,KT,H
      READ(3,*,END=160) (N(I),I = 1,IG)
      DO 140 I = 1,M
         READ(3,*,END=150) Y(I)
         X(I) = Y(I)
  140 CONTINUE
      GO TO 170
  150 WRITE(*,*) 'ERROR:  END-OF FILE ENCOUNTERED'
  160 GO TO 2000
  170 V1 = ZERO
      V2 = ZERO
      V3 = ZERO
      DO 175 I = 1,M
         V1 = V1+Y(I)
         V2 = V2+Y(I)**2
  175 CONTINUE
      NC(1) = N(1)
      V(1) = ZERO
      DO 180 I = 2,IG
         NC(I) = NC(I-1)+N(I)
         V(I) = ZERO
  180 CONTINUE
      DO 185 I = 1,NC(1)
         V(1) = V(1)+Y(I)
  185 CONTINUE
      V3 = V(1)**2/N(1)
      DO 195 J = 2,IG
         DO 190 I = NC(J-1)+1,NC(J)
            V(J) = V(J)+Y(I)
  190    CONTINUE
         V3 = V3+V(J)**2/N(J)
  195 CONTINUE
      DFN = ONE*(IG-1)
      DFD = ONE*(M-IG)
      FSTAT = ((V3-V1**2/M)/DFN)/((V2-V3)/DFD)
      IF(FSTAT.LT.1.0D-5) THEN
         PF = ONE
         GO TO 200
      END IF
      CALL FPV(FSTAT,DFN,DFD,PF)
  200 IF(IT.EQ.0) GO TO 600
C  M UNIVARIATE VALUES ARE REPLACED WITH TIE ADJUSTED RANK ORDER STATISTICS
      CL = 1.0D30
      DO 210 I = 1,M
         IF(Y(I).LT.CL) CL = Y(I)
  210 CONTINUE
      DO 215 I = 1,M
         Y(I) = Y(I)-CL+1
  215 CONTINUE
      A1 = ZERO
      A2 = ZERO
      A3 = ONE*M-0.1D0
      B1 = ZERO-1.0D30
      B2 = 1.0D30
  218 IF(A2.GT.A3) GO TO 222
      DO 219 I = 1,M
         IF(Y(I).GT.B1.AND.Y(I).LT.B2) B2 = Y(I)*(ONE+PHI)
  219 CONTINUE
      DO 220 I = 1,M
         W = DABS(ONE-Y(I)/B2)
         IF(W.LT.1.0D-11) A1 = A1+1
  220 CONTINUE
      A4 = A2+(A1+1)/2
      DO 221 I = 1,M
         W = DABS(ONE-Y(I)/B2)
         IF(W.LT.1.0D-11) R(I) = A4
  221 CONTINUE
      A2 = A2+A1
      A1 = ZERO
      B1 = B2
      B2 = 1.0D30
      GO TO 218
  222 Z = ONE*(M+1)/2
      DO 300 I = 1,M
         GO TO (225,230,235) KT
  225    X(I) = R(I)**H
         GO TO 300
  230    W = DABS(Z-R(I))
         X(I) = ZERO
         IF(W.LT.1.0D-11) GO TO 300
         X(I) = DABS(R(I)-Z)**H
         GO TO 300
  235    W = DABS(Z-R(I))
         X(I) = ZERO
         IF(W.LT.1.0D-11) GO TO 300
         IF(R(I).GT.Z) THEN
            X(I) = (R(I)-Z)**H
         ELSEIF(R(I).LT.Z) THEN
            X(I) = ZERO-(Z-R(I))**H
         ENDIF
  300 CONTINUE
  600 SUM1 = ZERO
      SUM2 = ZERO
      DO 729 I = 1,M
         SUM1 = SUM1+X(I)
         SUM2 = SUM2+X(I)**2
  729 CONTINUE
      T = ZERO
      K = 0
      DO 731 I = 1,IG
         S1(I) = ZERO
         DO 730 J = K+1,K+N(I)
            S1(I) = S1(I)+X(J)
  730    CONTINUE
         T = T+S1(I)**2/N(I)
         K = K+N(I)
  731 CONTINUE
      TKW = T*(ONE-PHI)
      MP = 0
      DO 750 IW = 1,MS
         CALL SAMPWOR(M,M,U,X)
         T = ZERO
         K = 0
         DO 741 I = 1,IG
            S1(I) = ZERO
            DO 740 J = K+1,K+N(I)
               S1(I) = S1(I)+X(J)
  740       CONTINUE
            T = T+S1(I)**2/N(I)
            K = K+N(I)
  741    CONTINUE
         IF(T.GT.TKW) MP = MP+1
  750 CONTINUE
      P = ONE*MP/MS
      TG = (TKW-SUM1**2/M)*(ONE*M*(M-1)/(SUM2*M-SUM1**2))
      IF(IT.EQ.0) THEN
         HEAD = 'RESAMPLING VERSION OF G-SAMPLE FISHER-PITMAN TEST'
         WRITE(6,800) HEAD
         GO TO 1100
      ENDIF
  800 FORMAT(T12,A)
      IF(IT.EQ.1) THEN
         GO TO (810,820,830) KT
  810    HEAD = 'RESAMPLING VERSION OF G-SAMPLE A(M,H) RANK TEST'
         WRITE(6,800) HEAD
         WRITE(6,900) 'VALUE OF H FOR A(M,H) RANK TEST =',H
         GO TO 850
  820    HEAD = 'RESAMPLING VERSION OF G-SAMPLE B(M,H) RANK TEST'
         WRITE(6,800) HEAD
         WRITE(6,900) 'VALUE OF H FOR B(M,H) RANK TEST =',H
         GO TO 850
  830    HEAD = 'RESAMPLING VERSION OF G-SAMPLE C(M,H) RANK TEST'
         WRITE(6,800) HEAD
         WRITE(6,900) 'VALUE OF H FOR C(M,H) RANK TEST =',H
  850 ENDIF
  900 FORMAT(T2,A,G16.8)
 1100 WRITE(6,1200) 'INPUT HAS ',M,' UNIVARIATE OBSERVATIONS'
 1200 FORMAT(T2,A,I5,A)
      WRITE(6,1300) 'THERE ARE ',IG, ' GROUPS OF SIZES ',
     +              (N(I),I = 1,IG)
 1300 FORMAT(T2,A,I2,A,10I5)
      WRITE(6,1400) 'RANDOM NUMBER GENERATOR SEED =',ISEED
 1400 FORMAT(T2,A,I9)
      WRITE(6,1400) 'NUMBER OF SIMULATIONS =',MS
      WRITE(6,1400) 'NUMBER OF UPPER TAIL VALUES =',MP
      WRITE(6,900) 'THE OBSERVED TEST STATISTIC =',TKW
      WRITE(6,900) 'STANDARDIZED CHI-SQUARED TEST STATISTIC =',TG
      WRITE(6,900) 'ONE-WAY ANOVA TEST STATISTIC =',FSTAT
      CALL CHISQ(TG,IG-1,PA)
      WRITE(6,900) 'UPPER TAIL RESAMPLING P-VALUE =',P
      WRITE(6,900) 'UPPER TAIL ASYMPTOTIC P-VALUE =',PA
      WRITE(6,900) 'UPPER TAIL ONE-WAY ANOVA P-VALUE =',PF
      WRITE(6,1500)
 1500 FORMAT('********************************************************')
      GO TO 130
 2000 CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE SAMPWOR(M,N,U,X)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17),X(5000)
C
C     M: SIZE OF POPULATION
C     N: SIZE OF SAMPLE DRAWN WITHOUT REPLACEMENT (1 </= N </= M)
C     DATA: REAL INPUT WHICH CONTAINS X(1),...,X(M) POPULATION VALUES
C     WITHOUT REPLACEMENT SAMPLE IS CONTAINED IN X(1),...,X(N)
C
C     UNI(U) IS UNIFORM RANDOM NUMBER GENERATOR ON (0,1)
C
      DO 10 I = 1,N
         J = INT(I+(M-I+1)*UNI(U))
         BIN = X(J)
         X(J) = X(I)
         X(I) = BIN
   10 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
C
C
      SUBROUTINE CHISQ(T,N,P)
      DOUBLE PRECISION T,ZERO,ONE,V,P,PI,A,B,E1,E2,E3,E4,E5,H,W
C
      ZERO = 0.0D0
      ONE = 1.0D0
      IF(T.LT.ZERO) THEN
         P = ONE
         GO TO 60
      END IF
      V = ONE*N
      IF(V/2-N/2.GT.0.1D0) GO TO 30
      P = DEXP(-T/2)
      IF(N.LT.3) GO TO 60
      K = (N-2)/2
      B = ONE
      DO 20 J = 1,K
         L = J
         A = ONE
         DO 10 I = 1,L
            A = A*T/(2*I)
   10    CONTINUE
         B = B+A
   20 CONTINUE
      P = P*B
      GO TO 60
   30 PI = DATAN(ONE)*4
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*DSQRT(T)+1)
      P = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*DEXP(-T/2)/DSQRT(PI/2)
      IF(N.LT.2) GO TO 60
      K = (N-1)/2
      B = ZERO
      DO 50 J = 1,K
         L = J
         A = ONE
         DO 40 I = 1,L
            A = A*T/(2*I-1)
   40    CONTINUE
         B = B+A
   50 CONTINUE
      P = P+B*DEXP(-T/2)/DSQRT(PI*T/2)
   60 RETURN
      END
C
C
      SUBROUTINE FPV(X,DN,DD,P)
      DOUBLE PRECISION  ONE,X,Y,DN,DD,A,B,P,BETAIN
C
      ONE = 1.0D0
      A = DN/2
      B = DD/2
      Y = A*X/(A*X+B)
      P = ONE - BETAIN(Y,A,B)
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION BETAIN(X, A, B)
      DOUBLE PRECISION  X,A,B,ONE,TOL,AB,CX,XX,AA,BB,TERM,AI,RX,TEMP,
     +       DLNBTA
      LOGICAL INDEX
      ONE = 1.0D0
      TOL = 0.1D-7
      BETAIN = X
      AB = A + B
      CX = ONE - X
      IF (A .GE. AB * X) GO TO 10
      XX = CX
      CX = X
      AA = B
      BB = A
      INDEX = .TRUE.
      GO TO 20
   10 XX = X
      AA = A
      BB = B
      INDEX = .FALSE.
   20 TERM = ONE
      AI = ONE
      BETAIN = ONE
      NS = IDINT(BB + CX * AB)
      RX = XX / CX
   30 TEMP = BB - AI
      IF (NS .EQ. 0) RX = XX
   40 TERM = TERM * TEMP * RX / (AA + AI)
      BETAIN = BETAIN + TERM
      TEMP = DABS(TERM)
      IF (TEMP .LE. TOL .AND. TEMP .LE. TOL * BETAIN) GO TO 50
      AI = AI + 1
      NS = NS - 1
      IF (NS .GE. 0) GO TO 30
      TEMP = AB
      AB = AB + 1
      GO TO 40
   50 BETAIN = BETAIN * DEXP(AA * DLOG(XX) + (BB - 1) * DLOG(CX) -
     +         DLNBTA(AA, BB)) / AA
      IF (INDEX) BETAIN = ONE - BETAIN
      END
C
      DOUBLE PRECISION FUNCTION DLNBTA(A, B)
      DOUBLE PRECISION  A,B,DLNGAM
      DLNBTA = DLNGAM(A) + DLNGAM(B) - DLNGAM(A + B)
      END
C
C
      DOUBLE PRECISION FUNCTION DLNGAM(X)
      DOUBLE PRECISION  X,ONE,Z,D,PI,F
      ONE  = 1.0D0
      Z  = X - 1
      D  = ONE
      PI = DATAN(ONE)*4
      F  = Z + 10
      DO 10 J = 1, 9
         D = D * (Z + J)
   10 CONTINUE
      DLNGAM = (F * 2 - 1) * DLOG(F) / 2 - F + DLOG(PI * 2) / 2
     +          - DLOG(D) + ONE / (F * 12) - ONE / (F**3 * 360)
     +          + ONE / (F**5 * 1260) - ONE / (F**7 * 1680)
     +          + ONE / (F**9 * 1188)
      END
