      PROGRAM RMRSP
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THE TEST STATISTIC ASSOCIATED WITH MULTI-RESPONSE SEQUENCE
C         PERMUTATION PROCEDURES (MRSP) IS CALCULATED IN THIS FORTRAN-77
C         PROGRAM.  THIS TEST IS DESIGNED TO ANALYZE MULTI-RESPONSE DATA
C         AT THE ORDINAL (OR HIGHER) LEVELS AND IS BASED ON THE AVERAGE
C         EUCLIDEAN DISTANCE BETWEEN ADJACENT POINTS OF AN OBSERVED SEQUENCE
C         OF POINTS IN AN R-DIMENSIONAL SPACE (WHICH CORRESPOND TO R MEASURED
C         RESPONSES ON EACH OBJECT OF AN OBSERVED SEQUENCE OF OBJECTS).
C         THE PRESENT MAXIMUM VALUES OF N AND R IN THIS PROGRAM ARE 200
C         AND 50, RESPECTIVELY.  AVERAGE DISTANCE FUNCTION COMMENSURATION
C         IS AN ALTERNATIVE.  THIS PROGRAM IS CAPABLE OF MULTIPLE RUNS.
C
C         PROGRAM MODIFIED 8/21/2006
C
      CHARACTER  HEAD*80,FIN*40,FOUT*40
      DIMENSION  U(17),AD(50),DATA(200,50)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA:'
      READ(*,100) FIN
  100 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,100) FOUT
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 104 II = 1,17
         S = 0.0D0
         T = 0.5D0
         DO 102 JJ = 1,K
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) S = S+T
            T = T/2
  102    CONTINUE
         U(II) = S
  104 CONTINUE
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
 1010 READ(3,100,END=1020) HEAD
      GO TO 1030
 1020 GO TO 2000
 1030 READ(3,*) NCASES,ID,V,IC
      DO 1040 I = 1,NCASES
         READ(3,*,END=1050) (DATA(I,J),J = 1,ID)
 1040 CONTINUE
      GO TO 1070
 1050 WRITE(*,1060) 'ERROR:  END OF FILE ENCOUNTERED'
 1060 FORMAT(/,T3,A)
      STOP
 1070 IF(ID.EQ.1) IC = 0
      IF(ID.EQ.1.OR.IC.NE.1) GO TO 1090
      DO 1076 I = 1,ID
         AD(I) = ZERO
         DO 1074 J = 1,N-1
            DO 1072 K = J+1,N
               AD(I) = AD(I)+(DABS(DATA(J,I)-DATA(K,I)))**V
 1072       CONTINUE
 1074    CONTINUE
         AD(I) = AD(I)**(ONE/V)
 1076 CONTINUE
      DO 1084 I = 1,N
         DO 1082 J = 1,ID
            DATA(I,J) = DATA(I,J)/AD(J)
 1082    CONTINUE
 1084 CONTINUE
 1090 CALL DIST(NCASES,ID,V,U,DATA,DBAR,PL,PU,MS)
      WRITE(6,100) HEAD
      HEAD = 'MULTI-RESPONSE PERMUTATION SEQUENCE PROCEDURES (MRSP)'
      WRITE(6,1100) HEAD
 1100 FORMAT(T23,A,/)
      WRITE(6,1200) 'INPUT HAS ', NCASES,' OBSERVATIONS'
 1200 FORMAT(T10,A,I3,A,/)
      WRITE(6,1200) 'THERE WERE', ID, ' DIMENSION(S)'
      WRITE(6,1400) 'RANDOM NUMBER GENERATOR SEED =',ISEED
 1400 FORMAT(T10,A,I9)
      WRITE(6,1400) 'NUMBER OF SIMULATIONS =',MS
      WRITE(6,1500) IC
 1500 FORMAT(T10,'1 (0) IMPLIES (NO) AVE. D.F. COMMENSURATION:',I3)
      WRITE(6,1800) 'THE DISTANCE EXPONENT =', V
 1800 FORMAT(T10,A,G16.8)
      WRITE(6,1800) 'THE OBSERVED DELTA IS =', DBAR
      WRITE(6,1800) 'LOWER TAIL MRSP P-VALUE =', PL
      WRITE(6,1800) 'UPPER TAIL MRSP P-VALUE =', PU
      WRITE(6,1850)
 1850 FORMAT('********************************************************')
      GO TO 1010
 2000 CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE DIST(NCASES,ID,V,U,DATA,DBAR,PL,PU,MS)
      IMPLICIT REAL*8(A-H,O-Z)
C
C         SUBROUTINE DIST CALCULATES THE DISTANCES BETWEEN THE DATA
C         POINTS, PLUS THE PARAMETERS WHICH ARE DIRECTLY DEPENDENT
C         ON THE DISTANCES.
C
      DIMENSION  U(17),DATA(200,50),DEL(200,200)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      DBAR = ZERO
      DO 1400 I = 1,NCASES
         DO 1300 J = 1,NCASES
            DEL(I,J) = ZERO
            DO 1200 K = 1,ID
               DEL(I,J) = DEL(I,J)+(DATA(I,K)-DATA(J,K))**2
 1200       CONTINUE
            DEL(I,J) = DEL(I,J)**(V/2)
 1300    CONTINUE
 1400 CONTINUE
      DO 1500 I = 1,NCASES-1
         DBAR = DBAR+DEL(I,I+1)
 1500 CONTINUE
      DBAR = DBAR/(NCASES-1)
      DL = DBAR*(ONE+1.0D-12)
      DU = DBAR*(ONE-1.0D-12)
      ML = 0
      MU = 0
      DO 2600 IW = 1,MS
         CALL SAMPWOR(NCASES,NCASES,ID,U,DATA)
         DZ = ZERO
         DO 2400 I = 1,NCASES
            DO 2300 J = 1,NCASES
               DEL(I,J) = ZERO
               DO 2200 K = 1,ID
                  DEL(I,J) = DEL(I,J)+(DATA(I,K)-DATA(J,K))**2
 2200          CONTINUE
               DEL(I,J) = DEL(I,J)**(V/2)
 2300       CONTINUE
 2400    CONTINUE
         DO 2500 I = 1,NCASES-1
            DZ = DZ+DEL(I,I+1)
 2500    CONTINUE
         DZ = DZ/(NCASES-1)
         IF(DZ.LT.DL) ML = ML+1
         IF(DZ.GT.DU) MU = MU+1
 2600 CONTINUE
      PL = ONE*ML/MS
      PU = ONE*MU/MS
      RETURN
      END
C
C
      SUBROUTINE SAMPWOR(M,N,ID,U,DATA)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17),BIN(50),DATA(200,50)
C
C     M: SIZE OF POPULATION
C     N: SIZE OF SAMPLE DRAWN WITHOUT REPLACEMENT (1 </= N </= M)
C     DATA: REAL INPUT WHICH CONTAINS DATA(1,*),...,DATA(M,*) POPULATION VALUES
C     WITHOUT REPLACEMENT SAMPLE CONTAINED IN DATA(1,*),...,DATA(N,*)
C
C     UNI(U) IS UNIFORM RANDOM NUMBER GENERATOR ON (0,1)
C
      DO 20 I = 1,N
         J = INT(I+(M-I+1)*UNI(U))
         DO 10 K = 1,ID
            BIN(K) = DATA(J,K)
            DATA(J,K) = DATA(I,K)
            DATA(I,K) = BIN(K)
   10    CONTINUE
   20 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END



