      PROGRAM RTERP
C
C     PROGRAM RTERP COMPUTES RESAMPLING VERSION OF THE TERPSTRA-JONCKHEERE
C     TEST FOR ORDERED ALTERNATIVES WHERE M(1),...,M(G) ARE G SAMPLE SIZES
C     AND WHERE N = M(1) + ... + M(G).  THE NULL HYPOTHESIS IS:
C                         H0: H(1) = H(2) ... = H(G)
C     AND THE ALTERNATIVE HYPOTHESIS IS:
C                         H1: H(1) <= H(2) ... <= H(G)
C     WHERE H REPRESENTS THE G SAMPLE MEDIANS.  THE RESAMPLING VERSION
C     OF THIS PROGRAM COMPESATES FOR TIED OBSERVATIONS WHEREAS THE NORMAL
C     APPROXIMATION ASSUMES THE DATA CONSIST OF CONTINUOUS UNIVARIATE
C     OBSERVATIONS WITH NO CONPENSATION FOR TIES AMONG THE N VALUES.
C
C     PROGRAM WRITTEN ON:  22 May 2000
C
C     THE CURRENT DIMENSIONS FOR PROGRAM TJTEST ARE SET FOR G = 10,
C     N = 1000, AND ANY M(I) <= 400.  TO CHANGE THE SIZE OF THE DIMENSIONS,
C     REPLACE THE VALUES IN THE 'DOUBLE PRECISION' AND THE 'INTEGER'
C     STATEMENTS BELOW.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION  U(17),M(11),Y(1000),X(400,11)
      CHARACTER  FIN*40,FOUT*40,HEAD*60
      PARAMETER  (CUTOFF = 3.0D5)
C
C  IF G > 10, N > 1000 OR AN M(I) > 400, REVISE THE ABOVE DIMENSION STATEMENT.
C
      ONE = 1.0D0
      HALF = 0.5D0
      ZERO = 0.0D0
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA'
      READ(*,'(A)') FIN
      WRITE(*,*) 'ENTER: FILE NAME FOR OUTPUT RESULTS'
      READ(*,'(A)') FOUT
      WRITE(*,*) 'ENTER: PROBLEM HEADING (UP TO 60 CHARACTERS)'
      READ(*,'(A)') HEAD
      WRITE(*,*) 'ENTER: G, THE NUMBER OF SAMPLES'
      READ(*,*) NG
      WRITE(*,*) 'ENTER: [M(1),...,M(G)], THE NUMBER OF OBSERVATIONS ',
     &           'IN EACH SAMPLE'
      READ(*,*) (M(I),I=1,NG)
      M(NG+1) = 0
      IF(NG.LT.2) THEN
         WRITE(*,*) 'PROGRAM TERMINATED SINCE G IS LESS THAN 2'
         GO TO 600
      ENDIF
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 4 II = 1,17
         S = 0.0D0
         T = 0.5D0
         DO 2 JJ = 1,K
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) S = S+T
            T = T/2
    2    CONTINUE
         U(II) = S
    4 CONTINUE
      N = 0
      L2 = 0
      L3 = 0
      DO 10 I = 1,NG
        N = N+M(I)
        L2 = L2+M(I)**2
        L3 = L3+M(I)**3
   10 CONTINUE
      AN = ONE*N
      AN2 = ONE*L2
      AN3 = ONE*L3
      EW = (AN**2-AN2)/4.0D0
      VW = (AN*AN*(AN*2+3)-AN3*2-AN2*3)/72.0D0
      OPEN(5,FILE=FIN,STATUS='OLD')
      READ(5,*,END=580) (Y(I),I = 1,N)
      CLOSE(5)
   17 K = 0
      DO 19 I = 1,NG
        DO 18 J = 1,M(I)
          K = K+1
          X(J,I) = Y(K)
   18   CONTINUE
   19 CONTINUE
C
C     CALCULATE OBSERVED TERPSTRA-JONCKHERE W STATISTIC
C
      W1 = ZERO
      DO 25 I = 1,NG-1
         DO 24 J = I+1,NG
            DO 23 K = 1,M(I)
               DO 22 L = 1,M(J)
                  IF(X(K,I).LT.X(L,J)) W1 = W1+ONE
                  IF(X(K,I).EQ.X(L,J)) W1 = W1+HALF
   22          CONTINUE
   23       CONTINUE
   24    CONTINUE
   25 CONTINUE
      W1C = W1-0.5D0
      Z = (W1C-EW)/DSQRT(VW)
      CALL PVALUE(Z,PVAL1)
C
C     CALCULATION OF RESAMPLED P-VALUE
C
      W1 = W1*(ONE-1.0D-12)
      MP = 0
      DO 100 IW = 1,MS
         CALL SAMPWOR(N,N,U,Y)
         K = 0
         DO 30 I = 1,NG
            DO 29 J = 1,M(I)
               K = K+1
               X(J,I) = Y(K)
   29       CONTINUE
   30    CONTINUE
         W = ZERO
         DO 50 I = 1,NG-1
            DO 45 J = I+1,NG
               DO 40 K = 1,M(I)
                  DO 35 L = 1,M(J)
                     IF(X(K,I).LT.X(L,J)) W = W+ONE
                     IF(X(K,I).EQ.X(L,J)) W = W+HALF
   35             CONTINUE
   40          CONTINUE
   45       CONTINUE
   50    CONTINUE
         IF(W.GT.W1) MP = MP+1
  100 CONTINUE
      PVAL2 = ONE*MP/MS
      OPEN(6,FILE=FOUT,STATUS='NEW')
      HEAD = 'TERPSTRA-JONCKHEERE (T-J) TEST'
      WRITE(6,150) HEAD
  150 FORMAT(T5,A,/)
      WRITE(6,200) 'INPUT HAS ', N,' UNIVARIATE OBSERVATIONS'
  200 FORMAT(T5,A,I3,A,/)
      WRITE(6,250) 'THERE ARE', NG, ' GROUPS OF SIZES ',
     +              (M(I),I = 1,NG)
  250 FORMAT(T5,A,I2,A,10I5)
      WRITE(6,300) 'RANDOM NUMBER GENERATOR SEED =',ISEED
  300 FORMAT(T5,A,I9)
      WRITE(6,300) 'NUMBER OF SIMULATIONS = ',MS
      WRITE(6,400) 'THE OBSERVED T-J TEST STATISTIC = ',W1
  400 FORMAT(T5,A,G16.8)
      WRITE(6,400) 'UPPER TAIL RESAMPLING P-VALUE = ',PVAL2
      WRITE(6,400) 'MEAN OF TERPSTRA-JONCKHEERE STATISTIC = ',EW
      WRITE(6,400) 'VARIANCE OF T-J STATISTIC = ',VW
      WRITE(6,400) 'UPPER TAIL NORMAL APPROXIMATE P-VALUE = ',PVAL1
      WRITE(6,500)
  500 FORMAT('********************************************************')
      CLOSE(6)
      GO TO 600
  580 WRITE(*,590) FIN
  590 FORMAT(/,T2,'E R R O R:  END-OF-FILE ENCOUNTERED: ',A,/)
  600 END
C
C
      SUBROUTINE PVALUE(Z,PROB)
C
C     SUBROUTINE PVALUE CALCULATES THE PROBABILITY THAT A VALUE OF
C     Z IS GREATER THAN OR EQUAL TO THE OBSERVED VALUE OF Z.
C
      DOUBLE PRECISION  Z,PROB,ZERO,ONE,PI,A,B,C,G,H,U,W,E1,E2,E3,E4,E5
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-738.0D0) GO TO 20
      PI = DATAN(ONE)*4
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 10
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*U+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*DEXP(G)/DSQRT(PI*2)
      GO TO 20
   10 B = 2.2D0
      C = 4.9D0
      PROB = DEXP(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   20 IF(Z.LT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      SUBROUTINE SAMPWOR(M,N,U,X)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17),X(1000)
C
C     M: SIZE OF POPULATION
C     N: SIZE OF SAMPLE DRAWN WITHOUT REPLACEMENT (1 </= N </= M)
C     DATA: REAL INPUT WHICH CONTAINS X(1),...,X(M) POPULATION VALUES
C     WITHOUT REPLACEMENT SAMPLE IS CONTAINED IN X(1),...,X(N)
C
C     UNI(U) IS UNIFORM RANDOM NUMBER GENERATOR ON (0,1)
C
      DO 10 I = 1,N
         J = INT(I+(M-I+1)*UNI(U))
         BIN = X(J)
         X(J) = X(I)
         X(I) = BIN
   10 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
