      PROGRAM RTSLT
      IMPLICIT REAL*8(A-H,O-Z)
C
C  THIS IS A RESAMPLING PROGRAM FOR THE TWO-SAMPLE FISHER-PITMAN TEST (IT = 0)
C  AND THE CLASSES OF LINEAR RANK TESTS (IT = 1).  WHEN IT = 1, THEN THE
C  CHOICE OF KT = 1, 2 OR 3 SELECTS THE CLASS OF A(M,H), B(M,H) OR C(M,H)
C  LINEAR RANK TESTS, RESPECTIVELY (SEE J. AMER. STATIST. ASSOC., VOL. 67,
C  PP. 850-854 FOR A DESCRIPTION OF THESE LINEAR RANK TESTS WHICH INCLUDE
C  THE MEDIAN, WILCOXON-MANN-WHITNEY, TAHA L, ANSARI-BRADLEY, MOOD AND
C  OTHER RANK TESTS).  EACH LINEAR RANK TEST IS ADJUSTED FOR TIED VALUES.
C  LET L BE THE NUMBER OF RESAMPLING SIMULATIONS. LET M BE THE TOTAL NUMBER
C  OF UNIVARIATE VALUES.  LET N1 & N2 BE THE NUMBER VALUES IN GROUPS 1 & 2,
C  RESPECTIVELY.  THE FIRST LINE OF INPUT DATA FILE CONSISTS OF THE ORDERED
C  VALUES M, N1, N2, IT, KT AND H, RESPECTIVELY.  THEN THE NEXT M LINES
C  CONSIST OF THE M UNIVARIATE VALUES WITH THE N1 VALUE OF GROUP 1 FOLLOWED
C  BY THE N2 VALUES OF GROUP 2.  ALSO INCLUDED ARE THE CORRESPONDING ASYMPTOTIC
C  AND CLASSICAL TWO-SAMPLE t TEST P-VALUES ARE GIVEN FOR COMPARATIVE PURPOSES.
C
C  NOTE:  THIS PROGRAM IS CAPABLE OF MULTIPLE RUNS (MAKE SURE THAT THERE ARE
C         NO SPACES BETWEEN THE MULTIPLE DATA SETS).
C
C         PROGRAM MODIFIED 5/2/2002
C
      CHARACTER  HEAD*70,FIN*40,FOUT*40
      DIMENSION  U(17),Y(5000),X(5000),R(5000)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PHI = 1.0D-12
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA:'
      READ(*,100) FIN
  100 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,100) FOUT
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 104 II = 1,17
         S = 0.0D0
         T = 0.5D0
         DO 102 JJ = 1,K
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) S = S+T
            T = T/2
  102    CONTINUE
         U(II) = S
  104 CONTINUE
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
  130 READ(3,*,END=160) M,N1,N2,IT,KT,H
      DO 140 I = 1,M
         READ(3,*,END=150) Y(I)
         X(I) = Y(I)
  140 CONTINUE
      S1 = ZERO
      SS1 = ZERO
      S2 = ZERO
      SS2 = ZERO
      DO 142 I = 1,N1
         S1 = S1+Y(I)
         SS1 = SS1+Y(I)**2
  142 CONTINUE
      DO 144 I = N1+1,M
         S2 = S2+Y(I)
         SS2 = SS2+Y(I)**2
  144 CONTINUE
      DF = ONE*(N1+N2-2)
      SSP = (SS1-S1**2/N1+SS2-S2**2/N2)/DF
      TS = (S1/N1-S2/N2)/DSQRT(SSP*(ONE/N1+ONE/N2))
      TSX = TS
      CALL TPV(TSX,DF,PT)
      PT = PT/2
      IF(S1/N1.GT.S2/N2) THEN
         PT1 = PT
         PT2 = ONE-PT
      ELSE
         PT1 = ONE-PT
         PT2 = PT
      END IF
      PT3 = DMIN1(PT1*2,PT2*2)
      GO TO 170
  150 WRITE(*,*) 'ERROR:  END-OF FILE ENCOUNTERED'
  160 GO TO 2000
  170 IF(IT.EQ.0) GO TO 600
C  M UNIVARIATE VALUES ARE REPLACED WITH TIE ADJUSTED RANK ORDER STATISTICS
      CL = 1.0D30
      DO 210 I = 1,M
         IF(Y(I).LT.CL) CL = Y(I)
  210 CONTINUE
      DO 215 I = 1,M
         Y(I) = Y(I)-CL+1
  215 CONTINUE
      A1 = ZERO
      A2 = ZERO
      A3 = ONE*M-0.1D0
      B1 = ZERO-1.0D30
      B2 = 1.0D30
  218 IF(A2.GT.A3) GO TO 222
      DO 219 I = 1,M
         IF(Y(I).GT.B1.AND.Y(I).LT.B2) B2 = Y(I)*(ONE+PHI)
  219 CONTINUE
      DO 220 I = 1,M
         W = DABS(ONE-Y(I)/B2)
         IF(W.LT.1.0D-11) A1 = A1+1
  220 CONTINUE
      A4 = A2+(A1+1)/2
      DO 221 I = 1,M
         W = DABS(ONE-Y(I)/B2)
         IF(W.LT.1.0D-11) R(I) = A4
  221 CONTINUE
      A2 = A2+A1
      A1 = ZERO
      B1 = B2
      B2 = 1.0D30
      GO TO 218
  222 Z = ONE*(M+1)/2
      DO 300 I = 1,M
         GO TO (225,230,235) KT
  225    X(I) = R(I)**H
         GO TO 300
  230    W = DABS(Z-R(I))
         X(I) = ZERO
         IF(W.LT.1.0D-11) GO TO 300
         X(I) = DABS(R(I)-Z)**H
         GO TO 300
  235    W = DABS(Z-R(I))
         X(I) = ZERO
         IF(W.LT.1.0D-11) GO TO 300
         IF(R(I).GT.Z) THEN
            X(I) = (R(I)-Z)**H
         ELSEIF(R(I).LT.Z) THEN
            X(I) = ZERO-(Z-R(I))**H
         ENDIF
  300 CONTINUE
  600 YM = 1.0D30
      ST = ZERO
      SST = ZERO
      DO 610 I = 1,M
         IF(DABS(X(I)).LT.YM) YM = DABS(X(I))
         ST = ST+X(I)
         SST = SST+X(I)**2
  610 CONTINUE
      SUM = ZERO
      DO 620 I = 1,N1
         SUM = SUM+X(I)
  620 CONTINUE
      TA = (SUM/N1-ST/M)*M/DSQRT((SST*M-ST**2)*N2/N1/(M-1))
      WW = DABS(TA)
      CALL NORMAL(WW,PZ)
      IF(TA.GT.ZERO) THEN
         PZ1 = PZ
         PZ2 = ONE-PZ
      ELSE
         PZ1 = ONE-PZ
         PZ2 = PZ
      END IF
      PZ3 = DMIN1(PZ1*2,PZ2*2)
      T1 = SUM-YM*1.0D-5
      T2 = SUM+YM*1.0D-5
      MP1 = 0
      MP2 = 0
      DO 750 IW = 1,MS
         CALL SAMPWOR(M,M,U,X)
         T = ZERO
         DO 700 I = 1,N1
            T = T+X(I)
  700    CONTINUE
         IF(T.GT.T1) MP1 = MP1+1
         IF(T.LT.T2) MP2 = MP2+1
  750 CONTINUE
      P1 = ONE*MP1/MS
      P2 = ONE*MP2/MS
      P3 = DMIN1(P1*2,P2*2,ONE)
      IF(IT.EQ.0) THEN
         HEAD = 'RESAMPLING VERSION OF 2-SAMPLE FISHER-PITMAN TEST'
         WRITE(6,800) HEAD
         GO TO 1100
      ENDIF
  800 FORMAT(T12,A)
      IF(IT.EQ.1) THEN
         GO TO (810,820,830) KT
  810    HEAD = 'RESAMPLING VERSION OF 2-SAMPLE A(M,H) RANK TEST'
         WRITE(6,800) HEAD
         WRITE(6,900) 'VALUE OF H FOR A(M,H) RANK TEST = ',H
         GO TO 850
  820    HEAD = 'RESAMPLING VERSION OF 2-SAMPLE B(M,H) RANK TEST'
         WRITE(6,800) HEAD
         WRITE(6,900) 'VALUE OF H FOR B(M,H) RANK TEST = ',H
         GO TO 850
  830    HEAD = 'RESAMPLING VERSION OF 2-SAMPLE C(M,H) RANK TEST'
         WRITE(6,800) HEAD
         WRITE(6,900) 'VALUE OF H FOR C(M,H) RANK TEST = ',H
  850 ENDIF
  900 FORMAT(T2,A,G16.8)
 1100 WRITE(6,1200) 'INPUT HAS ',M,' UNIVARIATE OBSERVATIONS'
 1200 FORMAT(T2,A,I5,A)
      WRITE(6,1300) 'THERE ARE TWO GROUPS OF SIZES ',N1,' AND ',N2
 1300 FORMAT(T2,A,I5,A,I5)
      WRITE(6,1400) 'RANDOM NUMBER GENERATOR SEED = ',ISEED
 1400 FORMAT(T2,A,I9)
      WRITE(6,1400) 'NUMBER OF SIMULATIONS = ',MS
      WRITE(6,1400) 'NUMBER OF UPPER TAIL VALUES = ',MP1
      WRITE(6,1400) 'NUMBER OF LOWER TAIL VALUES = ',MP2
      WRITE(6,900) 'THE OBSERVED SUM OF GROUP 1 VALUES = ',SUM
      WRITE(6,900) 'THE UPPER TAIL RESAMPLING P-VALUE = ',P1
      WRITE(6,900) 'THE LOWER TAIL RESAMPLING P-VALUE = ',P2
      WRITE(6,900) 'THE TWO-SIDED RESAMPLING P-VALUE = ',P3
      WRITE(6,900) 'THE OBSERVED ASYMPTOTIC STATISTIC = ',TA
      WRITE(6,900) 'THE UPPER TAIL ASYMPTOTIC P-VALUE = ',PZ1
      WRITE(6,900) 'THE LOWER TAIL ASYMPTOTIC P-VALUE = ',PZ2
      WRITE(6,900) 'THE TWO-SIDED ASYMPTOTIC P-VALUE = ',PZ3
      WRITE(6,1500) 'THE MEANS OF GROUPS 1 AND 2 ARE ',
     +              S1/N1,' AND ',S2/N2
 1500 FORMAT(T5,A,G16.8,A,G16.8)
      WRITE(6,900) 'POOLED STANDARD DEVIATION = ',DSQRT(SSP)
      WRITE(6,1600) 'THE DEGREES-OF-FREEDOM = ',N1+N2-2
 1600 FORMAT(T5,A,I4)
      WRITE(6,900) 'THE TWO-SAMPLE STUDENT t-TEST STATISTIC = ',TS
      WRITE(6,900) 'THE UPPER TAIL P-VALUE = ', PT1
      WRITE(6,900) 'THE LOWER TAIL P-VALUE = ', PT2
      WRITE(6,900) 'THE TWO-SIDED P-VALUE = ', PT3
      WRITE(6,1700)
 1700 FORMAT('********************************************************')
      GO TO 130
 2000 CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE SAMPWOR(M,N,U,X)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17),X(5000)
C
C     M: SIZE OF POPULATION
C     N: SIZE OF SAMPLE DRAWN WITHOUT REPLACEMENT (1 </= N </= M)
C     DATA: REAL INPUT WHICH CONTAINS X(1),...,X(M) POPULATION VALUES
C     WITHOUT REPLACEMENT SAMPLE IS CONTAINED IN X(1),...,X(N)
C
C     UNI(U) IS UNIFORM RANDOM NUMBER GENERATOR ON (0,1)
C
      DO 10 I = 1,N
         J = INT(I+(M-I+1)*UNI(U))
         BIN = X(J)
         X(J) = X(I)
         X(I) = BIN
   10 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
C
C
      SUBROUTINE TPV(X,DD,P)
      DOUBLE PRECISION ZERO,ONE,X,Y,DN,DD,A,B,P,BETAIN
C
      ZERO = 0.0D0
      ONE = 1.0D0
      X = X*X
      DN = ONE
      IF(DD.LE.ZERO) THEN
         STOP 'DF IS NOT POSITIVE'
      END IF
      IF(X.GT.ZERO) THEN
         GO TO 20
      ELSE
         P = ONE
         GO TO 30
      END IF
   20 A = DN/2
      B = DD/2
      Y = A*X/(A*X+B)
      P = ONE-BETAIN(Y,A,B)
   30 RETURN
      END
C
      DOUBLE PRECISION FUNCTION BETAIN(X,A,B)
      DOUBLE PRECISION  X,A,B,ONE,TOL,AB,CX,XX,AA,BB,TERM,AI,RX,TEMP,
     +       DLNBTA
      LOGICAL INDEX
      ONE = 1.0D0
      TOL = 0.1D-7
      BETAIN = X
      AB = A+B
      CX = ONE-X
      IF(A.GE.AB*X) GO TO 10
      XX = CX
      CX = X
      AA = B
      BB = A
      INDEX = .TRUE.
      GO TO 20
   10 XX = X
      AA = A
      BB = B
      INDEX = .FALSE.
   20 TERM = ONE
      AI = ONE
      BETAIN = ONE
      NS = IDINT(BB+CX*AB)
      RX = XX/CX
   30 TEMP = BB-AI
      IF(NS.EQ.0) RX = XX
   40 TERM = TERM*TEMP*RX/(AA+AI)
      BETAIN = BETAIN+TERM
      TEMP = DABS(TERM)
      IF(TEMP.LE.TOL.AND.TEMP.LE.TOL*BETAIN) GO TO 50
      AI = AI+1
      NS = NS-1
      IF(NS.GE.0) GO TO 30
      TEMP = AB
      AB = AB+1
      GO TO 40
   50 BETAIN = BETAIN*DEXP(AA*DLOG(XX)+(BB-1)*DLOG(CX)-DLNBTA(AA,BB))/AA
      IF(INDEX) BETAIN = ONE-BETAIN
      END
C
      DOUBLE PRECISION FUNCTION DLNBTA(A,B)
      DOUBLE PRECISION  A,B,DLNGAM
      DLNBTA = DLNGAM(A)+DLNGAM(B)-DLNGAM(A+B)
      END
C
C
      DOUBLE PRECISION FUNCTION DLNGAM(X)
      DOUBLE PRECISION  X,ONE,Z,D,PI,F
      ONE = 1.0D0
      Z = X-1
      D = ONE
      PI = DATAN(ONE)*4
      F = Z+10
      DO 10 J = 1,9
         D = D*(Z+J)
   10 CONTINUE
      DLNGAM = (F*2-1)*DLOG(F)/2-F+DLOG(PI*2)/2-DLOG(D)+ONE/(F*12)
     +         -ONE/((F**3)*360)+ONE/((F**5)*1260)-ONE/((F**7)*1680)
     +         +ONE/((F**9)*1188)
      END
C
C
      SUBROUTINE NORMAL(Z,PROB)
C
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY OF A VALUE OF
C        Z BEING GREATER THAN OR EQUAL TO THE OBSERVED VALUE OF Z.
C
      DOUBLE PRECISION  Z,PROB,ZERO,ONE,PI,G,A,B,C,E,H,
     +   U,W,E1,E2,E3,E4,E5
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-709.1D0) GO TO 20
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 10
      E1 =  0.31938153D0
      E2 = -0.356563782D0
      E3 =  1.781477937D0
      E4 = -1.821255978D0
      E5 =  1.330274429D0
      H  =  0.2316419D0
      W  =  ONE/(H*DABS(Z)+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 20
   10 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   20 IF(Z.LT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
C
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF(G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END
