      PROGRAM MRBP
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THIS FORTRAN PROGRAM COMPUTES THE TEST STATISTIC AND ASSOCIATED
C         P-VALUE FOR AN ANALYSIS OF A RANDOMIZED BLOCK EXPERIMENT (MRBP3).
C         THE CORRESPONDENCE BETWEEN A CORRELATION ANALYSIS AND A
C         RANDOMIZED BLOCK EXPERIMENT CAN BE USED TO GET A CORRELATION
C         COEFFICIENT AS WELL. THE MAXIMUM VALUES OF G, B AND R CAN BE
C         CHANGED FOR ANY EXAMPLE.  THE PRESENT MAXIMUM VALUES OF G, B
C         AND R IN THIS PROGRAM ARE RESPECTIVELY 10, 12 AND 15.
C
C       THIS PROGRAM IS CAPABLE OF PERFORMING (1) REPEATED MRBP ANALYSES
C       IN ONE OPERATION, (2) ALIGNMENT WITHIN BLOCKS, (3) DISTANCE
C       FUNCTION COMMENSURATION, AND (4) C(G,H) RANKS TEST.
C
C         PROGRAM MODIFIED 2/9/2003
C
C       THE DATA MATRIX MUST BE IN THE FOLLOWING SEQUENCE WITH EACH
C       OBJECT'S R RESPONSE VALUES ON A SEPARATE LINE AS FOLLOWS:
C
C         A(1,1,1),A(1,1,2),...,A(1,1,R)
C         A(1,2,1),A(1,2,2),...,A(1,2,R)
C                  ...
C         A(1,B,1),A(1,B,2),...,A(1,B,R)
C         A(2,1,1),A(2,1,2),...,A(2,1,R)
C                  ...
C         A(2,B,1),A(2,B,2),...,A(2,B,R)
C                  ...
C         A(G,1,1),A(G,1,2),...,A(G,1,R)
C                  ...
C         A(G,B,1),A(G,B,2),...,A(G,B,R)
C
C   THE INPUT DATA ARE IN FREE FORMAT.  SPECIFICALLY, V IS DISTANCE EXPONENT,
C   G IS # OF GROUPS, B IS # OF BLOCKS, R IS # OF RESPONSES, IA = 1 IMPLIES
C   ALIGNMENT, IC = 1 IMPLIES COMMENSURATION, AND LR = 1 IMPLIES C(G,H)
C   RANKS TEST.   NOTE: ASSOCIATE G, B AND R WITH KG, KB AND KR IN PROGRAM.
C
      CHARACTER HEAD*80,FIN*40,FOUT*40
      DIMENSION AD(15),XM(12,15),DATA(10,12,15),X(10,12,15)
C
      ONE=1.0D0
      ZERO=0.0D0
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA'
      READ(*,'(A)') FIN
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,'(A)') FOUT
      OPEN(5,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
    2 READ(5,'(A)',END=5) HEAD
      GO TO 10
    5 GO TO 100
   10 READ(5,*) V,KG,KB,KR,IA,IC,LR,H
      DO 20 I=1,KG
         DO 15 J=1,KB
            READ(5,*,END=75) (DATA(I,J,K),K=1,KR)
   15    CONTINUE
   20 CONTINUE
      WRITE(6,21) HEAD,V,KR,KB,KG
   21 FORMAT(' ANALYSIS OF RANDOMIZED BLOCK EXPERIMENT (MRBP): '/
     1     4X,A/4X,'DISTANCE FUNCTION EXPONENT: ',F5.2/4X,'WITH ',I3,
     2     ' RESPONSES, ',I3,' BLOCKS, ',I3,' GROUPS.')
      IF(LR.EQ.1) THEN
         IA = 0
         IC = 0
         CALL RANK(KG,KB,KR,H,DATA)
         GO TO 60
      END IF
      IF(IA.EQ.0) GO TO 40
      DO 25 I=1,KG
         DO 24 J=1,KB
            DO 23 K=1,KR
               X(I,J,K)=DATA(I,J,K)
   23       CONTINUE
   24    CONTINUE
   25 CONTINUE
      DO 30 J=1,KB
         DO 29 K=1,KR
            A1=1.0D200
            SUM1=A1
            DO 28 I1=1,KG
               SUM=ZERO
               DO 27 I2=1,KG
                  SUM=SUM+DABS(DATA(I2,J,K)-X(I1,J,K))
   27          CONTINUE
               IF(SUM.LT.A1) THEN
                  DM1=X(I1,J,K)
                  A1=SUM
                  A2=A1*1.0000000001D0
               END IF
               IF(SUM.LT.A2.AND.DM1.NE.X(I1,J,K)) THEN
                  DM2=X(I1,J,K)
                  SUM1=SUM
               END IF
   28       CONTINUE
            IF(SUM1.GT.A2) DM2=DM1
            XM(J,K)=(DM1+DM2)/2
   29    CONTINUE
   30 CONTINUE
      DO 35 I=1,KG
         DO 34 J=1,KB
            DO 33 K=1,KR
               DATA(I,J,K)=DATA(I,J,K)-XM(J,K)
   33       CONTINUE
   34    CONTINUE
   35 CONTINUE
   40 IF(IC.EQ.0) GO TO 60
      IF(KR.EQ.1) GO TO 60
      DO 45 K=1,KR
         AD(K)=ZERO
         DO 44 I1=1,KG
            DO 43 I2=1,KG
               DO 42 J1=2,KB
                  DO 41 J2=1,J1-1
                     AD(K)=AD(K)+(DABS(DATA(I1,J1,K)-DATA(I2,J2,K)))**V
   41             CONTINUE
   42          CONTINUE
   43       CONTINUE
   44    CONTINUE
         AD(K)=AD(K)**(ONE/V)
   45 CONTINUE
      DO 50 I=1,KG
         DO 49 J=1,KB
            DO 48 K=1,KR
               DATA(I,J,K)=DATA(I,J,K)/AD(K)
   48       CONTINUE
   49    CONTINUE
   50 CONTINUE
   60 CALL CALC(V,KG,KB,KR,IA,IC,LR,H,DATA)
      GO TO 2
   75 WRITE(6,80)
   80 FORMAT(' E R R O R:  END-OF-FILE ENCOUNTERED.')
      WRITE(6,90) KG,KB,KR
   90 FORMAT(' CHECK THESE:  G = ',I3,' B = ',I3,' R = ',I3)
      STOP
  100 CLOSE(5)
      CLOSE(6)
      END
C
C
      SUBROUTINE RANK(KG,KB,KR,H,DATA)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION RKS(10),DATA(10,12,15)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      YM = ONE*(KG+1)/2
      DO 80 J = 1,KB
         DO 70 K = 1,KR
            CL = 1.0D30
            DO 3 I = 1,KG
               IF(DATA(I,J,K).LT.CL) CL = DATA(I,J,K)
    3       CONTINUE
            DO 7 I = 1,KG
               DATA(I,J,K) = DATA(I,J,K)-CL+ONE
    7       CONTINUE
            PHI = ONE+1.0D-12
            A1 = ZERO
            A2 = ZERO
            A3 = ONE*KG-0.1D0
            B1 = ZERO-1.0D30
            B2 = 1.0D30
   10       IF(A2.GT.A3)  GO TO 50
            DO 20 I = 1,KG
               IF(DATA(I,J,K).GT.B1.AND.DATA(I,J,K).LT.B2)
     1           B2 = DATA(I,J,K)*PHI
   20       CONTINUE
            DO 30 I = 1,KG
               W = DABS(ONE-DATA(I,J,K)/B2)
               IF(W.LT.1.0D-11)  A1 = A1+1
   30       CONTINUE
            A4 = A2+(A1+1)/2
            DO 40 I = 1,KG
               W = DABS(ONE-DATA(I,J,K)/B2)
               IF(W.LT.1.0D-11)  RKS(I) = A4
   40       CONTINUE
            A2 = A2+A1
            A1 = ZERO
            B1 = B2
            B2 = 1.0D30
            GO TO 10
   50       DO 60 I = 1,KG
               W = DABS(RKS(I)-YM)
               IF(W.LT.0.001D0)  THEN
                  DATA(I,J,K) = ZERO
               ELSE
                  DATA(I,J,K) = (RKS(I)-YM)*DABS(RKS(I)-YM)**(H-1)
               END IF
   60       CONTINUE
   70    CONTINUE
   80 CONTINUE
      RETURN
      END
C
C
      SUBROUTINE CALC(V,KG,KB,KR,IA,IC,LR,H,DATA)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION D(120,120),SIJ(12,12),SIJ2(12,12),SIJ3(12,12),
     1    UIJ(12,12),TIJ2(12,12),TIJ3(12,12),VI(12,12),DATA(10,12,15),
     2    SJ(10,12,12),SJ2(10,12,12),SJ3(10,12,12),UJ(10,12,12),
     3    WI(12,12,12),YIJ(12,12,12),ZIJK(12,12,12)
C
      ONE=1.0D0
      ZERO=0.0D0
      Y=ONE*V/2
      BC2=ONE*KB*(KB-1)/2
      KBG=KB*KG
      DO 4 I=1,KBG
         DO 3 J=1,KBG
            D(I,J)=ZERO
    3    CONTINUE
    4 CONTINUE
      DO 10 I=1,KG
         DO 9 J=1,KB
            DO 8 K=I,KG
               LO=1
               IF(I.EQ.K) LO=J
               DO 7 L=LO,KB
                  IJ=KB*(I-1)+J
                  KL=KB*(K-1)+L
                  D(IJ,KL)=ZERO
                  DO 6 M=1,KR
                     D(IJ,KL)=D(IJ,KL)+(DATA(I,J,M)-DATA(K,L,M))**2
    6             CONTINUE
                  D(IJ,KL)=D(IJ,KL)**Y
                  D(KL,IJ)=D(IJ,KL)
    7          CONTINUE
    8       CONTINUE
    9    CONTINUE
   10 CONTINUE
      DO 30 IS=1,KB
         DO 29 IR=1,KB
            IF(IR.EQ.IS) GO TO 29
            SIJ(IR,IS)=ZERO
            SIJ2(IR,IS)=ZERO
            SIJ3(IR,IS)=ZERO
            DO 28 I=1,KG
               SJ(I,IR,IS)=ZERO
               SJ2(I,IR,IS)=ZERO
               SJ3(I,IR,IS)=ZERO
               DO 27 J=1,KG
                  IRR=(I-1)*KB+IR
                  JSS=(J-1)*KB+IS
                  SJ(I,IR,IS)=SJ(I,IR,IS)+D(IRR,JSS)
                  SJ2(I,IR,IS)=SJ2(I,IR,IS)+D(IRR,JSS)**2
                  SJ3(I,IR,IS)=SJ3(I,IR,IS)+D(IRR,JSS)**3
   27          CONTINUE
               SIJ(IR,IS)=SIJ(IR,IS)+SJ(I,IR,IS)
               SIJ2(IR,IS)=SIJ2(IR,IS)+SJ2(I,IR,IS)
               SIJ3(IR,IS)=SIJ3(IR,IS)+SJ3(I,IR,IS)
   28       CONTINUE
   29    CONTINUE
   30 CONTINUE
      T2=ZERO
      IF(KB.LE.2) GO TO 51
      DO 50 IT=3,KB
         IT1=IT-1
         DO 49 IS=2,IT1
            IS1=IS-1
            DO 48 IR=1,IS1
               WI(IR,IS,IT)=ZERO
               WI(IS,IR,IT)=ZERO
               WI(IT,IR,IS)=ZERO
               YIJ(IR,IS,IT)=ZERO
               ZIJK(IR,IS,IT)=ZERO
               DO 47 I=1,KG
                  WI(IR,IS,IT)=WI(IR,IS,IT)+SJ(I,IR,IS)*SJ(I,IR,IT)
                  WI(IS,IR,IT)=WI(IS,IR,IT)+SJ(I,IS,IR)*SJ(I,IS,IT)
                  WI(IT,IR,IS)=WI(IT,IR,IS)+SJ(I,IT,IR)*SJ(I,IT,IS)
                  DO 46 J=1,KG
                     IRR=KB*(I-1)+IR
                     JSS=KB*(J-1)+IS
                     JTT=KB*(J-1)+IT
                     ISS=KB*(I-1)+IS
                     YIJ(IR,IS,IT)=YIJ(IR,IS,IT)+
     1                 D(IRR,JSS)*SJ(I,IR,IT)*SJ(J,IS,IT)+
     2                 D(IRR,JTT)*SJ(I,IR,IS)*SJ(J,IT,IS)+
     3                 D(ISS,JTT)*SJ(I,IS,IR)*SJ(J,IT,IR)
                     DO 45 K=1,KG
                        KTT=(K-1)*KB+IT
                        ZIJK(IR,IS,IT)=ZIJK(IR,IS,IT)+
     1                    D(IRR,JSS)*D(IRR,KTT)*D(JSS,KTT)
   45                CONTINUE
   46             CONTINUE
   47          CONTINUE
               T2=T2+SIJ(IR,IS)*SIJ(IR,IT)*SIJ(IS,IT)-
     1           SIJ(IS,IT)*WI(IR,IS,IT)*KG-
     2           SIJ(IR,IT)*WI(IS,IR,IT)*KG-
     3           SIJ(IR,IS)*WI(IT,IR,IS)*KG+
     4           YIJ(IR,IS,IT)*KG*KG-ZIJK(IR,IS,IT)*KG*KG*KG
   48       CONTINUE
   49    CONTINUE
   50 CONTINUE
      T2=T2*6/(KG-1)
   51 DO 100 IS=2,KB
         IS1=IS-1
         DO 99 IR=1,IS1
            TIJ2(IR,IS)=ZERO
            TIJ3(IR,IS)=ZERO
            VI(IR,IS)=ZERO
            UIJ(IR,IS)=ZERO
            DO 98 I=1,KG
               TIJ2(IR,IS)=TIJ2(IR,IS)+SJ(I,IR,IS)**2+SJ(I,IS,IR)**2
               TIJ3(IR,IS)=TIJ3(IR,IS)+SJ(I,IR,IS)**3+SJ(I,IS,IR)**3
               VI(IR,IS)=VI(IR,IS)+SJ(I,IR,IS)*SJ2(I,IR,IS)+
     1           SJ(I,IS,IR)*SJ2(I,IS,IR)
               UJ(I,IR,IS)=ZERO
               DO 97 J=1,KG
                  IRR=KB*(I-1)+IR
                  JSS=KB*(J-1)+IS
                  UJ(I,IR,IS)=UJ(I,IR,IS)+
     1              D(IRR,JSS)*SJ(I,IR,IS)*SJ(J,IS,IR)
   97          CONTINUE
               UIJ(IR,IS)=UIJ(IR,IS)+UJ(I,IR,IS)
   98       CONTINUE
   99    CONTINUE
  100 CONTINUE
      T1=ZERO
      EDEL=ZERO
      VAR=ZERO
      DO 200 IS=2,KB
         IS1=IS-1
         DO 199 IR=1,IS1
            IF(KG.LE.2) GO TO 198
            T1=T1+(SIJ(IR,IS)**3)*4-SIJ(IR,IS)*TIJ2(IR,IS)*KG*6+
     1        UIJ(IR,IS)*KG*KG*6+TIJ3(IR,IS)*KG*KG*2+
     2        SIJ(IR,IS)*SIJ2(IR,IS)*KG*KG*3-
     3        VI(IR,IS)*KG*KG*KG*3+SIJ3(IR,IS)*KG*KG*KG*KG
  198       EDEL=EDEL+SIJ(IR,IS)
            VAR=VAR+SIJ(IR,IS)*SIJ(IR,IS)-TIJ2(IR,IS)*KG+
     1        SIJ2(IR,IS)*KG*KG
  199    CONTINUE
  200 CONTINUE
      IF(KG.LE.2) GO TO 206
      T1=T1/(KG-2)
  206 C1=ONE/BC2/KG/KG
      C2=C1*C1
      C3=C2*C1
      EDEL=C1*EDEL
      VAR=VAR*C2/(KG-1)
      GAM=C3*(T1-T2)/(KG-1)
      GAM=GAM/DSQRT(VAR**3)
      DELTA=ZERO
      DO 300 IS=2,KB
         IS1=IS-1
         DO 299 IR=1,IS1
            DO 298 I=1,KG
               IRR=(I-1)*KB+IR
               ISS=(I-1)*KB+IS
               DELTA=DELTA+D(IRR,ISS)
  298       CONTINUE
  299    CONTINUE
  300 CONTINUE
      C0=BC2*KG
      DELTA=DELTA/C0
      T=(DELTA-EDEL)/DSQRT(VAR)
      RHO=ONE-(DELTA/EDEL)
      WRITE(6,580)
  580 FORMAT(/'   RESULTS OF MRBP ANALYSIS:')
      WRITE(6,585) LR
  585 FORMAT(11X,'1 (0) INDICATES (NO) C(G,H) RANKS TEST:',I3)
      IF(LR.EQ.1) THEN
        WRITE(6,588) H
  588   FORMAT(11X,'C(G,H) RANKS TEST EXPONENT H =',G16.7)
      ELSE
        WRITE(6,590)
  590   FORMAT(' ')
      END IF
      WRITE(6,593) IA
  593 FORMAT(11X,'1 (0) INDICATES (NO) MEDIAN ALIGNMENT:',I3)
      IF(KR.EQ.1) THEN
        WRITE(6,590)
      ELSE
        WRITE(6,597) IC
  597   FORMAT(11X,'1 (0) INDICATES (NO) AVE. DIST. COMMENSURATION:',I3)
      END IF
      WRITE(6,600) DELTA
  600 FORMAT(11X,'DELTA(D)',T29,':  ',G16.7)
      WRITE(6,610) EDEL
  610 FORMAT(11X,'EXPECTED DELTA',T29,':  ',G16.7)
      WRITE(6,620) VAR
  620 FORMAT(11X,'VARIANCE OF DELTA',T29,':  ',G16.7)
      WRITE(6,630) GAM
  630 FORMAT(11X,'SKEWNESS OF DELTA',T29,':  ',G16.7)
      WRITE(6,640) RHO
  640 FORMAT(11X,'AGREEMENT MEASURE',T29,':  ',G16.7)
      WRITE(6,650) T
  650 FORMAT(11X,'(D-E[D])/STDEV(D)',T29,':  ',G16.7)
      CALL PVALUE(T,GAM,PROB)
      WRITE(6,670) PROB
  670 FORMAT(11X,'P-VALUE',T29,':  ',G16.7)
      WRITE(6,680)
  680 FORMAT('********************************************************')
      RETURN
      END
C
C
      SUBROUTINE PVALUE(T,GAM,PROB)
      IMPLICIT REAL*8(A-H,O-Z)
C
C         SUBROUTINE PVALUE CALCULATES THE PROBABILITY THAT A VALUE OF
C         T IS LESS THAN OR EQUAL TO THE OBSERVED VALUE OF T.
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      IF(GAM.GT.-0.01D0) GO TO 50
      R = ONE*2/DABS(GAM)
      D = R*R
      F = R*R+10
      DO 10 I = 1,9
         D = D*(R*R+I)
   10 CONTINUE
      U = (F*2-1)*DLOG(F)/2-F+DLOG(PI*2)/2-DLOG(D)+ONE/(F*12)
     +        -ONE/(F*F*F*360)
      A = R*R-1
      B = R*R*(DLOG(R)-1)-U
      G1 = 0.0375D0
      G2 = 0.075D0
      G3 = 0.0125D0
      H1 = ZERO
      H2 = ZERO
      W = -1.99D0/GAM
      IF(GAM.LT.ZERO) GO TO 30
      IF(T.LT.W) THEN
         PROB = ZERO
         RETURN
      END IF
      X = T
      Y = T+15
      DO 20 I = 1,199
         PHI = A*DLOG(R+X+G1*(2*I-1))-R*(X+G1*(2*I-1))+B
         H1 = H1+E(PHI)
         PHI = A*DLOG(R+X+G2*I)-R*(X+G2*I)+B
         H2 = H2+E(PHI)
   20 CONTINUE
      PHI = A*DLOG(R+X+G1*399)-R*(X+G1*399)+B
      H1N = E(PHI)
      PHI = A*DLOG(R+X)-R*X+B
      H0 = E(PHI)
      PHI = A*DLOG(R+Y)-R*Y+B
      H3 = E(PHI)
      PROB = ONE-G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   30 IF(T.GT.W) THEN
         PROB = ONE
         RETURN
      END IF
      X = T-15
      Y = T
      DO 40 I = 1,199
         PHI = A*DLOG(R-X-G1*(2*I-1))+R*(X+G1*(2*I-1))+B
         H1 = H1+E(PHI)
         PHI = A*DLOG(R-X-G2*I)+R*(X+G2*I)+B
         H2 = H2+E(PHI)
   40 CONTINUE
      PHI = A*DLOG(R-X-G1*399)+R*(X+G1*399)+B
      H1N = E(PHI)
      PHI = A*DLOG(R-X)+R*X+B
      H0 = E(PHI)
      PHI = A*DLOG(R-Y)+R*Y+B
      H3 = E(PHI)
      PROB = G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   50 PROB = ZERO
      G = -T*T/2
      IF(G.LT.-709.1D0) GO TO 70
      A = 4.2D0
      U = DABS(T)
      IF(U.GT.A) GO TO 60
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*U+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 70
   60 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   70 IF(T.GT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
      IMPLICIT REAL*8(A-H,O-Z)
C
      E = 0.0D0
      IF(G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END
