      PROGRAM EIGEN
      IMPLICIT REAL*8(A-H,O-Z)
C
C  PROGRAM EIGEN IS APPROPRIATE ONLY FOR SQUARE SYMMETRIC MATRICES
C  WHERE THE LOWER TRIANGULAR PORTION OF A GIVEN MATRIX IS USED
C  IN THE ACTUAL COMPUTATIONS OF THE INVERSE MATRIX.
C
      CHARACTER  FOUT*40,FMT1*20
      DIMENSION  A(20,20),D(20),E(20),Z(20,20)
      DATA  FMT1 /'(/,X(6F13.6,/))'/
C
      ND = 20
      ETA = 1.0D-200
      PRECIS = 1.0D-12
      TOL = ETA/PRECIS
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,50) FOUT
   50 FORMAT(A)
      OPEN(6,FILE=FOUT,STATUS='NEW')
      WRITE(*,*) 'ENTER ORDER OF MATRIX'
      READ(*,*) N
      DO 100 I = 1,N
         WRITE(*,*) 'ENTER ROW ', I, ' OF THE MATRIX'
         READ(*,*) (A(I,J),J=1,N)
  100 CONTINUE
      WRITE(6,50) '      ****** SPECTRAL DECOMPOSITION ANALYSIS ******'
      K = 0
      DO 110 I = 1,43,6
         IF(N.GE.I) K = K+1
  110 CONTINUE
      WRITE(UNIT=FMT1(4:4),FMT='(I1)') K
      WRITE(6,120) 'ORIGINAL DATA MATRIX:'
  120 FORMAT(//,A)
      DO 130 I = 1,N
         WRITE(6,FMT1) (A(I,J),J=1,N)
  130 CONTINUE
C
C        CALL SUBROUTINES TDIAG AND LRVT
C
      CALL TDIAG(N,ND,TOL,A,D,E,Z)
      CALL LRVT(N,ND,PRECIS,D,E,Z)
C
      WRITE(6,120) 'LIST OF EIGENVALUES:'
      WRITE(6,FMT1) (D(I),I=1,N)
      WRITE(6,120) 'EIGENVECTOR MATRIX (BY COLUMNS):'
      DO 140 I = 1,N
         WRITE(6,FMT1) (Z(I,J),J=1,N)
  140 CONTINUE
      END
C
C
      SUBROUTINE TDIAG(N,ND,TOL,A,D,E,Z)
      IMPLICIT REAL*8(A-H,O-Z)
C
C        REDUCES REAL SYMMETRIC MATRIX TO TRIDIAGONAL FORM
C
      DIMENSION  A(ND,N),D(N),E(N),Z(ND,N)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      DO 110 I = 1,N
         DO 100 J = 1,I
            Z(I,J) = A(I,J)
  100    CONTINUE
  110 CONTINUE
      I = N
      DO 200 I1 = 2,N
         L = I-2
         F = Z(I,I-1)
         G = ZERO
         IF(L.GE.1) THEN
            DO 120 K = 1,L
               G = G+Z(I,K)**2
  120       CONTINUE
         END IF
         H = G+F*F
C
C           IF G IS TOO SMALL FOR ORTHOGONALITY TO BE GUARANTEED,
C           THE TRANSFORMATION IS SKIPPED
C
         IF(G.GT.TOL) GO TO 130
         E(I) = F
         D(I) = ZERO
         GO TO 190
  130    L = L+1
         G = DSQRT(H)
         IF(F.GE.ZERO) G = ZERO-G
         E(I) = G
         H = H-F*G
         Z(I,I-1) = F-G
         F = ZERO
         DO 160 J = 1,L
            Z(J,I) = Z(I,J)/H
            G = ZERO
C
C              FORM ELEMENT OF A*U
C
            DO 140 K = 1,J
               G = G+Z(J,K)*Z(I,K)
  140       CONTINUE
            IF(J.LT.L) THEN
               DO 150 K = J+1,L
                  G = G+Z(K,J)*Z(I,K)
  150          CONTINUE
            END IF
C
C              FORM ELEMENT OF P
C
            E(J) = G/H
            F = F+G*Z(J,I)
  160    CONTINUE
C
C           FORM K
C
         HH = F/(H+H)
C
C           FORM REDUCED A
C
         DO 180 J = 1,L
            F = Z(I,J)
            G = E(J)-HH*F
            E(J) = G
            DO 170 K = 1,J
               Z(J,K) = Z(J,K)-F*E(K)-G*Z(I,K)
  170       CONTINUE
  180    CONTINUE
         D(I) = H
  190    I = I-1
  200 CONTINUE
      D(1) = ZERO
      E(1) = ZERO
C
C        ACCUMULATION OF TRANSFORMATION MATRICES
C
      DO 260 I = 1,N
         L = I-1
         IF(D(I).EQ.ZERO.OR.L.EQ.0) GO TO 240
         DO 230 J = 1,L
            G = ZERO
            DO 210 K = 1,L
               G = G+Z(I,K)*Z(K,J)
  210       CONTINUE
            DO 220 K = 1,L
               Z(K,J) = Z(K,J)-G*Z(K,I)
  220       CONTINUE
  230    CONTINUE
  240    D(I) = Z(I,I)
         Z(I,I) = ONE
         IF(L.NE.0) THEN
            DO 250 J = 1,L
               Z(I,J) = ZERO
               Z(J,I) = ZERO
  250       CONTINUE
         END IF
  260 CONTINUE
      RETURN
      END
C
C
      SUBROUTINE LRVT(N,ND,PRECIS,D,E,Z)
      IMPLICIT REAL*8(A-H,O-Z)
C
C        FINDS LATENT ROOTS AND VECTORS OF TRIDIAGONAL MATRIX
C
      DIMENSION  D(N),E(N),Z(ND,N)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      MITS = 30
      DO 100 I = 2,N
         E(I-1) = E(I)
  100 CONTINUE
      E(N) = ZERO
      B = ZERO
      F = ZERO
      DO 180 L = 1,N
         J = 0
         H = PRECIS*(DABS(D(L))+DABS(E(L)))
         IF(B.LT.H) B = H
C
C           LOOK FOR SMALL SUB-DIAGONAL ELEMENT
C
         DO 110 M1 = L,N
            M = M1
            IF(DABS(E(M)).LE.B) GO TO 120
  110    CONTINUE
  120    IF(M.EQ.L) GO TO 170
  130    IF(J.EQ.MITS) RETURN
         J = J+1
C
C           FORM SHIFT
C
         P = (D(L+1)-D(L))/(E(L)*2)
         R = DSQRT(P*P+ONE)
         PR = P+R
         IF(P.LT.ZERO) PR = P-R
         H = D(L)-E(L)/PR
         DO 140 I = L,N
            D(I) = D(I)-H
  140    CONTINUE
         F = F+H
C
C           QL TRANSFORMATION
C
         P = D(M)
         C = ONE
         S = ZERO
         I = M
         DO 160 I1 = L,M-1
            J = I
            I = I-1
            G = C*E(I)
            H = C*P
            IF(DABS(P).GE.DABS(E(I))) THEN
               C = E(I)/P
               R = DSQRT(C*C+ONE)
               E(J) = S*P*R
               S = C/R
               C = ONE/R
            ELSE
               C = P/E(I)
               R = DSQRT(C*C+ONE)
               E(J) = S*E(I)*R
               S = ONE/R
               C = C/R
            END IF
            P = C*D(I)-S*G
            D(J) = H+S*(C*G+S*D(I))
C
C              FORM VECTOR
C
            DO 150 K = 1,N
               H = Z(K,J)
               Z(K,J) = S*Z(K,I)+C*H
               Z(K,I) = C*Z(K,I)-S*H
  150       CONTINUE
  160    CONTINUE
         E(L) = S*P
         D(L) = C*P
         IF(DABS(E(L)).GT.B) GO TO 130
  170    D(L) = D(L)+F
  180 CONTINUE
C
C        ORDER LATENT ROOTS AND VECTORS
C
      DO 210 I = 1,N-1
         K = I
         P = D(I)
         DO 190 J = I+1,N
            IF(D(J).GT.P) THEN
               K = J
               P = D(J)
            END IF
  190    CONTINUE
         IF(K.NE.I) THEN
            D(K) = D(I)
            D(I) = P
            DO 200 J = 1,N
               P = Z(J,I)
               Z(J,I) = Z(J,K)
               Z(J,K) = P
  200       CONTINUE
         END IF
  210 CONTINUE
      RETURN
      END
