      PROGRAM SRTC
      IMPLICIT REAL*8(A-H,O-Z)
C
C  THIS IS A RESAMPLING TREATMENT-COMPARED-TO-CONTROL RIDIT ANALYSIS
C  PROGRAM YIELDING LOWER AND UPPER TAIL P-VALUES.  TREATMENT AND
C  CONTROL FREQUENCIES FOR THE K DISCRETE CELLS ARE ENTERED
C  INTERACTIVELY (K < 51).  IN ADDITION TO THE VALUE OF K AND
C  THE TREATMENT AND CONTROL FREQUENCY DATA THAT MUST BE ENTERED,
C  A POSITIVE INTEGER SEED AND NUMBER OF SIMULATIONS ARE ALSO ENTERED.
C
C         PROGRAM MODIFIED 1/16/2006
C
      CHARACTER  FOUT*40
      DIMENSION  U(17),L(50),M(50),RC(50),V(50)
C
C  IF K > 50, REPLACE 50 WITH K IN L(K), M(K), RC(K) AND V(K) ABOVE.
C
      ZERO = 0.0D0
      ONE = 1.0D0
      WRITE(*,1) 'RESAMPLING TREATMENT-CONTROL RIDIT ANALYSIS'
    1 FORMAT(T2,A)
      WRITE(*,1) 'ENTER K, THE NUMBER OF CELLS:'
      READ(*,*) K
      V(1) = 1.0D0/DBLE(K)
      DO 2 I = 2,K-1
        V(I) = V(I-1)+V(1)
    2 CONTINUE
      V(K) = ONE
      WRITE(*,1) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,3) FOUT
    3 FORMAT(A)
      WRITE(*,1) 'ENTER CONTROL FREQUENCIES M1,...,MK:'
      READ(*,*) (M(I),I=1,K)
      WRITE(*,1) 'ENTER TREATMENT FREQUENCIES L1,...,LK:'
      READ(*,*) (L(I),I=1,K)
      WRITE(*,1) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      PHI = 1.0D-12
      LT = 0
      MC = 0
      DO 4 I = 1,K
        LT = LT+L(I)
        MC = MC+M(I)
        RC(I) = ZERO
    4 CONTINUE
      RC(1) = ONE*M(1)/(2*MC)
      DO 7 I = 2,K
        RC(I) = ONE*M(I)/(2*MC)
        DO 6 J = 1,I-1
          RC(I) = RC(I)+ONE*M(J)/MC
    6   CONTINUE
    7 CONTINUE
      XT = ZERO
      XC = ZERO
      DO 9 I = 1,K
        XT = XT+RC(I)*L(I)/LT
        XC = XC+RC(I)*M(I)/MC
    9 CONTINUE
      XTL = XT*(ONE+PHI)
      XTU = XT*(ONE-PHI)
C
      KK = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 104 II = 1,17
         S = 0.0D0
         T = 0.5D0
         DO 102 JJ = 1,KK
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) S = S+T
            T = T/2
  102    CONTINUE
         U(II) = S
  104 CONTINUE
C
      LL = 0
      LU = 0
      Y1 = ZERO
      Y2 = ZERO
      DO 250 IW = 1,MS
         DO 200 I = 1,K
            L(I) = 0
  200    CONTINUE
         DO 210 I = 1,LT
            W = UNI(U)
            DO 205 J = 1,K
               IF(W.LE.V(J)) THEN
                  L(J) = L(J)+1
                  GO TO 210
               END IF
  205       CONTINUE
  210    CONTINUE
         YT = ZERO
         DO 220 I = 1,K
            YT = YT+RC(I)*L(I)
  220    CONTINUE
         YT = YT/LT
         IF(YT.LT.XTL) LL = LL+1
         IF(YT.GT.XTU) LU = LU+1
         Y1 = Y1+YT
         Y2 = Y2+YT**2
  250 CONTINUE
      PL = ONE*LL/MS
      PU = ONE*LU/MS
      SE = DSQRT((Y2*MS-Y1**2)/MS/(MS-1))
      Y1 = Y1/MS
      Z = (XT-Y1)/SE
      CALL NORMAL(Z,PNL,PNU)
C
      OPEN(5,FILE=FOUT,STATUS='NEW')
      WRITE(5,310) 'RESAMPLING TREATMENT-CONTROL RIDIT ANALYSIS ',
     +             'WITH ',K,' CELLS'
      WRITE(5,330) 'NUMBER OF TREATMENT CASES = ',LT
      WRITE(5,330) 'NUMBER OF CONTROL CASES = ',MC
      WRITE(5,330) 'RANDOM NUMBER GENERATOR SEED = ',ISEED
      WRITE(5,330) 'TOTAL NUMBER OF SIMULATIONS = ',MS
      WRITE(5,330) 'NUMBER OF LOWER TAIL VALUES = ',LL
      WRITE(5,330) 'NUMBER OF UPPER TAIL VALUES = ',LU
      WRITE(5,320) XC, ' = CHECK THAT CONTROL RIDIT STATISTIC IS 0.5'
      WRITE(5,320) XT, ' = OBSERVED TREATMENT RIDIT STATISTIC'
      WRITE(5,320) Y1, ' = EXPECTED VALUE OF TREATMENT RIDIT STATISTIC'
      WRITE(5,320) SE, ' = STANDARD ERROR OF TREATMENT RIDIT STATISTIC'
      WRITE(5,320) Z, ' = STANDARDIZED TREATMENT RIDIT STATISTIC'
      WRITE(5,320) PNL, ' = ASYMPTOTIC LOWER TAIL RIDIT P-VALUE'
      WRITE(5,320) PNU, ' = ASYMPTOTIC UPPER TAIL RIDIT P-VALUE'
      WRITE(5,320) PL, ' = RESAMPLING LOWER TAIL RIDIT P-VALUE'
      WRITE(5,320) PU, ' = RESAMPLING UPPER TAIL RIDIT P-VALUE'
  310 FORMAT(/,T2,A,A,I3,A/)
  320 FORMAT(T5,D16.9,A)
  330 FORMAT(T5,A,I9)
      END
C.......................................................................
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
C.......................................................................
      SUBROUTINE NORMAL(Z,PNL,PNU)
C
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY OF A VALUE OF
C        Z BEING GREATER THAN OR EQUAL TO THE OBSERVED VALUE OF Z.
C
      DOUBLE PRECISION  Z,PNL,PNU,PROB,ZERO,ONE,PI,G,A,B,C,E,H,
     +   U,W,E1,E2,E3,E4,E5
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-709.1D0) GO TO 20
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 10
      E1 =  0.31938153D0
      E2 = -0.356563782D0
      E3 =  1.781477937D0
      E4 = -1.821255978D0
      E5 =  1.330274429D0
      H  =  0.2316419D0
      W  =  ONE/(H*DABS(Z)+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 20
   10 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   20 IF(Z.GT.ZERO) THEN
         PNU = PROB
         PNL = ONE-PROB
      ELSE
         PNL = PROB
         PNU = ONE-PROB
      END IF
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
C
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF(G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END

